@@ -12,17 +12,16 @@ use Encode;
 sub printheader ($) {
 	my $session=shift;
 	
-	if (($ENV{HTTPS} && lc $ENV{HTTPS} ne "off") || $config{sslcookie}) {
+	if ($config{sslcookie}) {
 		print $session->header(-charset => 'utf-8',
 			-cookie => $session->cookie(-httponly => 1, -secure => 1));
-	}
-	else {
+	} else {
 		print $session->header(-charset => 'utf-8',
 			-cookie => $session->cookie(-httponly => 1));
 	}
 }
 
-sub prepform {
+sub showform ($$$$;@) {
 	my $form=shift;
 	my $buttons=shift;
 	my $session=shift;
@@ -35,79 +34,13 @@ sub prepform {
 		});
 	}
 
-	return $form;
-}
-
-sub showform ($$$$;@) {
-	my $form=prepform(@_);
-	shift;
-	my $buttons=shift;
-	my $session=shift;
-	my $cgi=shift;
-
 	printheader($session);
-	print cgitemplate($cgi, $form->title,
-		$form->render(submit => $buttons), @_);
-}
-
-sub cgitemplate ($$$;@) {
-	my $cgi=shift;
-	my $title=shift;
-	my $content=shift;
-	my %params=@_;
-	
-	my $template=template("page.tmpl");
-
-	my $topurl = $config{url};
-	if (defined $cgi && ! $config{w3mmode} && ! $config{reverse_proxy}) {
-		$topurl = $cgi->url;
-	}
-
-	my $page="";
-	if (exists $params{page}) {
-		$page=delete $params{page};
-		$params{forcebaseurl}=urlto($page);
-		if (! $config{html5}) {
-			$params{forcebaseurl}=urlabs($params{forcebaseurl}, $topurl);
-		}
-	}
-	run_hooks(pagetemplate => sub {
-		shift->(
-			page => $page,
-			destpage => $page,
-			template => $template,
-		);
-	});
-	templateactions($template, "");
-
-	my $baseurl = baseurl();
-	if (! $config{html5}) {
-		$baseurl = urlabs($baseurl, $topurl),
-	}
-
-	$template->param(
-		dynamic => 1,
-		title => $title,
-		wikiname => $config{wikiname},
-		content => $content,
-		baseurl => $baseurl,
-		html5 => $config{html5},
-		%params,
-	);
-	
-	return $template->output;
+	print misctemplate($form->title, $form->render(submit => $buttons), @_);
 }
 
 sub redirect ($$) {
 	my $q=shift;
-	eval q{use URI};
-
-	my $topurl;
-	if (defined $q && ! $config{w3mmode} && ! $config{reverse_proxy}) {
-		$topurl = $q->url;
-	}
-
-	my $url=URI->new(urlabs(shift, $topurl));
+	my $url=shift;
 	if (! $config{w3mmode}) {
 		print $q->redirect($url);
 	}
@@ -118,35 +51,21 @@ sub redirect ($$) {
 }
 
 sub decode_cgi_utf8 ($) {
-	# decode_form_utf8 method is needed for 5.01
+	# decode_form_utf8 method is needed for 5.10
 	if ($] < 5.01) {
 		my $cgi = shift;
 		foreach my $f ($cgi->param) {
-			$cgi->param($f, map { decode_utf8 $_ }
-				@{$cgi->param_fetch($f)});
+			$cgi->param($f, map { decode_utf8 $_ } $cgi->param($f));
 		}
 	}
 }
 
-sub safe_decode_utf8 ($) {
-    my $octets = shift;
-    # call decode_utf8 on >= 5.20 only if it's not already decoded,
-    # otherwise it balks, on < 5.20, always call it
-    if ($] < 5.02 || !Encode::is_utf8($octets)) {
-        return decode_utf8($octets);
-    }
-    else {
-        return $octets;
-    }
-}
-
 sub decode_form_utf8 ($) {
 	if ($] >= 5.01) {
 		my $form = shift;
 		foreach my $f ($form->field) {
-			my @value=map { safe_decode_utf8($_) } $form->field($f);
 			$form->field(name  => $f,
-			             value => \@value,
+			             value => decode_utf8($form->field($f)),
 		                     force => 1,
 			);
 		}
@@ -161,17 +80,16 @@ sub needsignin ($$) {
 
 	if (! defined $session->param("name") ||
 	    ! userinfo_get($session->param("name"), "regdate")) {
-		$session->param(postsignin => $q->query_string);
+		$session->param(postsignin => $ENV{QUERY_STRING});
 		cgi_signin($q, $session);
 		cgi_savesession($session);
 		exit;
 	}
 }
 
-sub cgi_signin ($$;$) {
+sub cgi_signin ($$) {
 	my $q=shift;
 	my $session=shift;
-	my $returnhtml=shift;
 
 	decode_cgi_utf8($q);
 	eval q{use CGI::FormBuilder};
@@ -184,13 +102,16 @@ sub cgi_signin ($$;$) {
 		required => 'NONE',
 		javascript => 0,
 		params => $q,
-		action => cgiurl(),
+		action => $config{cgiurl},
 		header => 0,
 		template => {type => 'div'},
-		stylesheet => 1,
+		stylesheet => baseurl()."style.css",
 	);
 	my $buttons=["Login"];
 	
+	if ($q->param("do") ne "signin" && !$form->submitted) {
+		$form->text(gettext("You need to log in first."));
+	}
 	$form->field(name => "do", type => "hidden", value => "signin",
 		force => 1);
 	
@@ -205,11 +126,6 @@ sub cgi_signin ($$;$) {
 		$form->validate;
 	}
 
-	if ($returnhtml) {
-		$form=prepform($form, $buttons, $session, $q);
-		return $form->render(submit => $buttons);
-	}
-
 	showform($form, $buttons, $session, $q);
 }
 
@@ -266,9 +182,9 @@ sub cgi_prefs ($$) {
 		required => 'NONE',
 		javascript => 0,
 		params => $q,
-		action => cgiurl(),
+		action => $config{cgiurl},
 		template => {type => 'div'},
-		stylesheet => 1,
+		stylesheet => baseurl()."style.css",
 		fieldsets => [
 			[login => gettext("Login")],
 			[preferences => gettext("Preferences")],
@@ -299,11 +215,11 @@ sub cgi_prefs ($$) {
 	
 	if ($form->submitted eq 'Logout') {
 		$session->delete();
-		redirect($q, baseurl(undef));
+		redirect($q, $config{url});
 		return;
 	}
 	elsif ($form->submitted eq 'Cancel') {
-		redirect($q, baseurl(undef));
+		redirect($q, $config{url});
 		return;
 	}
 	elsif ($form->submitted eq 'Save Preferences' && $form->validate) {
@@ -315,20 +231,14 @@ sub cgi_prefs ($$) {
 		$form->text(gettext("Preferences saved."));
 	}
 	
-	showform($form, $buttons, $session, $q,
-		prefsurl => "", # avoid showing the preferences link
-	);
+	showform($form, $buttons, $session, $q);
 }
 
-sub cgi_custom_failure ($$$) {
-	my $q=shift;
-	my $httpstatus=shift;
+sub cgi_custom_failure ($$) {
+	my $header=shift;
 	my $message=shift;
 
-	print $q->header(
-		-status => $httpstatus,
-		-charset => 'utf-8',
-	);
+	print $header;
 	print $message;
 
 	# Internet Explod^Hrer won't show custom 404 responses
@@ -351,7 +261,7 @@ sub check_banned ($$) {
 
 	foreach my $b (@{$config{banned_users}}) {
 		if (pagespec_match("", $b,
-			ip => $session->remote_addr(),
+			ip => $ENV{REMOTE_ADDR},
 			name => defined $name ? $name : "",
 		)) {
 			$banned=1;
@@ -363,7 +273,7 @@ sub check_banned ($$) {
 		$session->delete();
 		cgi_savesession($session);
 		cgi_custom_failure(
-			$q, "403 Forbidden",
+			$q->header(-status => "403 Forbidden"),
 			gettext("You are banned."));
 	}
 }
@@ -381,8 +291,7 @@ sub cgi_getsession ($) {
 			{ FileName => "$config{wikistatedir}/sessions.db" })
 	};
 	if (! $session || $@) {
-		my $error = $@;
-		error($error." ".CGI::Session->errstr());
+		error($@." ".CGI::Session->errstr());
 	}
 	
 	umask($oldmask);
@@ -460,7 +369,7 @@ sub cgi (;$$) {
 			# userinfo db.
 			if (! userinfo_get($session->param("name"), "regdate")) {
 				userinfo_setall($session->param("name"), {
-					email => defined $session->param("email") ? $session->param("email") : "",
+					email => "",
 					password => "",
 					regdate => time,
 				}) || error("failed adding user");
@@ -492,7 +401,7 @@ sub cgierror ($) {
 	my $message=shift;
 
 	print "Content-type: text/html\n\n";
-	print cgitemplate(undef, gettext("Error"),
+	print misctemplate(gettext("Error"),
 		"<p class=\"error\">".gettext("Error").": $message</p>");
 	die $@;
 }
