@@ -19,7 +19,6 @@ sub getsetup () {
 		plugin => {
 			safe => 1,
 			rebuild => 0,
-			section => "web",
 		},
 		allowed_attachments => {
 			type => "pagespec",
@@ -58,7 +57,7 @@ sub check_canattach ($$;$) {
 			$config{allowed_attachments},
 			file => $file,
 			user => $session->param("name"),
-			ip => $session->remote_addr(),
+			ip => $ENV{REMOTE_ADDR},
 		);
 	}
 
@@ -113,7 +112,7 @@ sub formbuilder (@) {
 
 	return if ! defined $form->field("do") || ($form->field("do") ne "edit" && $form->field("do") ne "create") ;
 
-	my $filename=Encode::decode_utf8($q->param('attachment'));
+	my $filename=$q->param('attachment');
 	if (defined $filename && length $filename &&
             ($form->submitted eq "Upload Attachment" || $form->submitted eq "Save Page")) {
 		my $session=$params{session};
@@ -134,13 +133,10 @@ sub formbuilder (@) {
 			}
 		}
 
-		$filename=IkiWiki::basename($filename);
-		$filename=~s/.*\\+(.+)/$1/; # hello, windows
-
 		$filename=linkpage(IkiWiki::possibly_foolish_untaint(
 				attachment_location($form->field('page')).
-				$filename));
-		if (IkiWiki::file_pruned($filename)) {
+				IkiWiki::basename($filename)));
+		if (IkiWiki::file_pruned($filename, $config{srcdir})) {
 			error(gettext("bad attachment filename"));
 		}
 		
@@ -183,12 +179,9 @@ sub formbuilder (@) {
 		if ($config{rcs}) {
 			IkiWiki::rcs_add($filename);
 			IkiWiki::disable_commit_hook();
-			IkiWiki::rcs_commit(
-				file => $filename,
-				message => gettext("attachment upload"),
-				token => IkiWiki::rcs_prepedit($filename),
-				session => $session,
-			);
+			IkiWiki::rcs_commit($filename, gettext("attachment upload"),
+				IkiWiki::rcs_prepedit($filename),
+				$session->param("name"), $ENV{REMOTE_ADDR});
 			IkiWiki::enable_commit_hook();
 			IkiWiki::rcs_update();
 		}
@@ -196,19 +189,11 @@ sub formbuilder (@) {
 		IkiWiki::saveindex();
 	}
 	elsif ($form->submitted eq "Insert Links") {
-		my $page=quotemeta(Encode::decode_utf8($q->param("page")));
+		my $page=quotemeta($q->param("page"));
 		my $add="";
 		foreach my $f ($q->param("attachment_select")) {
-			$f=Encode::decode_utf8($f);
 			$f=~s/^$page\///;
-			if (IkiWiki::isinlinableimage($f) &&
-			    UNIVERSAL::can("IkiWiki::Plugin::img", "import")) {
-				$add.='[[!img '.$f.' align="right" size="" alt=""]]';
-			}
-			else {
-				$add.="[[$f]]";
-			}
-			$add.="\n";
+			$add.="[[$f]]\n";
 		}
 		$form->field(name => 'editcontent',
 			value => $form->field('editcontent')."\n\n".$add,
@@ -238,13 +223,13 @@ sub attachment_list ($) {
 	my @ret;
 	foreach my $f (values %pagesources) {
 		if (! defined pagetype($f) &&
-		    $f=~m/^\Q$loc\E[^\/]+$/) {
+		    $f=~m/^\Q$loc\E[^\/]+$/ &&
+		    -e "$config{srcdir}/$f") {
 			push @ret, {
 				"field-select" => '<input type="checkbox" name="attachment_select" value="'.$f.'" />',
 				link => htmllink($page, $page, $f, noimageinline => 1),
 				size => IkiWiki::Plugin::filecheck::humansize((stat(_))[7]),
 				mtime => displaytime($IkiWiki::pagemtime{$f}),
-				mtime_raw => $IkiWiki::pagemtime{$f},
 			};
 		}
 	}
