@@ -22,8 +22,10 @@ use warnings;
 use strict;
 use IkiWiki 3.00;
 use Time::Local;
-use POSIX ();
+use POSIX;
 
+my %cache;
+my %linkcache;
 my $time=time;
 my @now=localtime($time);
 
@@ -38,7 +40,6 @@ sub getsetup () {
 		plugin => {
 			safe => 1,
 			rebuild => undef,
-			section => "widget",
 		},
 		archivebase => {
 			type => "string",
@@ -47,14 +48,6 @@ sub getsetup () {
 			safe => 1,
 			rebuild => 1,
 		},
-		archive_pagespec => {
-			type => "pagespec",
-			example => "page(posts/*) and !*/Discussion",
-			description => "PageSpec of pages to include in the archives; used by ikiwiki-calendar command",
-			link => 'ikiwiki/PageSpec',
-			safe => 1,
-			rebuild => 0,
-		},
 }
 
 sub is_leap_year (@) {
@@ -74,56 +67,28 @@ sub month_days {
 sub format_month (@) {
 	my %params=@_;
 
-	my %linkcache;
-	foreach my $p (pagespec_match_list($params{page}, 
-				"creation_year($params{year}) and creation_month($params{month}) and ($params{pages})",
-				# add presence dependencies to update
-				# month calendar when pages are added/removed
-				deptype => deptype("presence"))) {
-		my $mtime = $IkiWiki::pagectime{$p};
-		my @date  = localtime($mtime);
-		my $mday  = $date[3];
-		my $month = $date[4] + 1;
-		my $year  = $date[5] + 1900;
-		my $mtag  = sprintf("%02d", $month);
-
-		# Only one posting per day is being linked to.
-		$linkcache{"$year/$mtag/$mday"} = $p;
-	}
-		
-	my $pmonth = $params{month} - 1;
-	my $nmonth = $params{month} + 1;
-	my $pyear  = $params{year};
-	my $nyear  = $params{year};
-
-	# Adjust for January and December
-	if ($params{month} == 1) {
-		$pmonth = 12;
-		$pyear--;
-	}
-	if ($params{month} == 12) {
-		$nmonth = 1;
-		$nyear++;
-	}
-
-	# Add padding.
-	$pmonth=sprintf("%02d", $pmonth);
-	$nmonth=sprintf("%02d", $nmonth);
+	my $pagespec = $params{pages};
+	my $year     = $params{year};
+	my $month    = $params{month};
+	my $pmonth   = $params{pmonth};
+	my $nmonth   = $params{nmonth};
+	my $pyear    = $params{pyear};
+	my $nyear    = $params{nyear};
 
+	my @list;
 	my $calendar="\n";
 
 	# When did this month start?
-	my @monthstart = localtime(timelocal(0,0,0,1,$params{month}-1,$params{year}-1900));
+	my @monthstart = localtime(timelocal(0,0,0,1,$month-1,$year-1900));
 
 	my $future_dom = 0;
 	my $today      = 0;
-	if ($params{year} == $now[5]+1900 && $params{month} == $now[4]+1) {
+	if ($year == $now[5]+1900 && $month == $now[4]+1) {
 		$future_dom = $now[3]+1;
 		$today      = $now[3];
 	}
 
 	# Find out month names for this, next, and previous months
-	my $monthabbrev=POSIX::strftime("%b", @monthstart);
 	my $monthname=POSIX::strftime("%B", @monthstart);
 	my $pmonthname=POSIX::strftime("%B", localtime(timelocal(0,0,0,1,$pmonth-1,$pyear-1900)));
 	my $nmonthname=POSIX::strftime("%B", localtime(timelocal(0,0,0,1,$nmonth-1,$nyear-1900)));
@@ -133,43 +98,34 @@ sub format_month (@) {
 	$archivebase = $params{archivebase} if defined $params{archivebase};
   
 	# Calculate URL's for monthly archives.
-	my ($url, $purl, $nurl)=("$monthname $params{year}",'','');
-	if (exists $pagesources{"$archivebase/$params{year}/$params{month}"}) {
+	my ($url, $purl, $nurl)=("$monthname",'','');
+	if (exists $cache{$pagespec}{"$year/$month"}) {
 		$url = htmllink($params{page}, $params{destpage}, 
-			"$archivebase/$params{year}/".$params{month},
-			noimageinline => 1,
-			linktext => "$monthabbrev $params{year}",
-			title => $monthname);
+			"$archivebase/$year/".sprintf("%02d", $month),
+			linktext => " $monthname ");
 	}
-	add_depends($params{page}, "$archivebase/$params{year}/$params{month}",
-		deptype("presence"));
-	if (exists $pagesources{"$archivebase/$pyear/$pmonth"}) {
+	add_depends($params{page}, "$archivebase/$year/".sprintf("%02d", $month));
+	if (exists $cache{$pagespec}{"$pyear/$pmonth"}) {
 		$purl = htmllink($params{page}, $params{destpage}, 
-			"$archivebase/$pyear/$pmonth",
-			noimageinline => 1,
-			linktext => "\&larr;",
-			title => $pmonthname);
+			"$archivebase/$pyear/" . sprintf("%02d", $pmonth),
+			linktext => " $pmonthname ");
 	}
-	add_depends($params{page}, "$archivebase/$pyear/$pmonth",
-		deptype("presence"));
-	if (exists $pagesources{"$archivebase/$nyear/$nmonth"}) {
+	add_depends($params{page}, "$archivebase/$pyear/".sprintf("%02d", $pmonth));
+	if (exists $cache{$pagespec}{"$nyear/$nmonth"}) {
 		$nurl = htmllink($params{page}, $params{destpage}, 
-			"$archivebase/$nyear/$nmonth",
-			noimageinline => 1,
-			linktext => "\&rarr;",
-			title => $nmonthname);
+			"$archivebase/$nyear/" . sprintf("%02d", $nmonth),
+			linktext => " $nmonthname ");
 	}
-	add_depends($params{page}, "$archivebase/$nyear/$nmonth",
-		deptype("presence"));
+	add_depends($params{page}, "$archivebase/$nyear/".sprintf("%02d", $nmonth));
 
 	# Start producing the month calendar
 	$calendar=<<EOF;
 <table class="month-calendar">
-	<tr>
-	<th class="month-calendar-arrow">$purl</th>
-	<th class="month-calendar-head" colspan="5">$url</th>
-	<th class="month-calendar-arrow">$nurl</th>
-	</tr>
+	<caption class="month-calendar-head">
+	$purl
+	$url
+	$nurl
+	</caption>
 	<tr>
 EOF
 
@@ -181,12 +137,12 @@ EOF
 	my %downame;
 	my %dowabbr;
 	for my $dow ($week_start_day..$week_start_day+6) {
-		my @day=localtime(timelocal(0,0,0,$start_day++,$params{month}-1,$params{year}-1900));
+		my @day=localtime(timelocal(0,0,0,$start_day++,$month-1,$year-1900));
 		my $downame = POSIX::strftime("%A", @day);
-		my $dowabbr = substr($downame, 0, 1);
+		my $dowabbr = POSIX::strftime("%a", @day);
 		$downame{$dow % 7}=$downame;
 		$dowabbr{$dow % 7}=$dowabbr;
-		$calendar.= qq{\t\t<th class="month-calendar-day-head $downame" title="$downame">$dowabbr</th>\n};
+		$calendar.= qq{\t\t<th class="month-calendar-day-head $downame">$dowabbr</th>\n};
 	}
 
 	$calendar.=<<EOF;
@@ -202,9 +158,9 @@ EOF
 
 	# At this point, either the first is a week_start_day, in which case
 	# nothing has been printed, or else we are in the middle of a row.
-	for (my $day = 1; $day <= month_days(year => $params{year}, month => $params{month});
+	for (my $day = 1; $day <= month_days(year => $year, month => $month);
 	     $day++, $wday++, $wday %= 7) {
-		# At this point, on a week_start_day, we close out a row,
+		# At tihs point, on a week_start_day, we close out a row,
 		# and start a new one -- unless it is week_start_day on the
 		# first, where we do not close a row -- since none was started.
 		if ($wday == $week_start_day) {
@@ -213,8 +169,8 @@ EOF
 		}
 		
 		my $tag;
-		my $key="$params{year}/$params{month}/$day";
-		if (defined $linkcache{$key}) {
+		my $mtag = sprintf("%02d", $month);
+		if (defined $cache{$pagespec}{"$year/$mtag/$day"}) {
 			if ($day == $today) {
 				$tag='month-calendar-day-this-day';
 			}
@@ -223,10 +179,9 @@ EOF
 			}
 			$calendar.=qq{\t\t<td class="$tag $downame{$wday}">};
 			$calendar.=htmllink($params{page}, $params{destpage}, 
-				$linkcache{$key},
-				noimageinline => 1,
-				linktext => $day,
-				title => pagetitle(IkiWiki::basename($linkcache{$key})));
+			                    pagename($linkcache{"$year/$mtag/$day"}),
+			                    "linktext" => "$day");
+			push @list, pagename($linkcache{"$year/$mtag/$day"});
 			$calendar.=qq{</td>\n};
 		}
 		else {
@@ -252,108 +207,102 @@ EOF
 </table>
 EOF
 
+	# Add dependencies to update the calendar whenever pages
+	# matching the pagespec are added or removed.
+	add_depends($params{page}, $params{pages});
+	# Explicitly add all currently linked pages as dependencies, so
+	# that if they are removed, the calendar will be sure to be updated.
+	foreach my $p (@list) {
+		add_depends($params{page}, $p);
+	}
+
 	return $calendar;
 }
 
 sub format_year (@) {
 	my %params=@_;
-	
-	my @post_months;
-	foreach my $p (pagespec_match_list($params{page}, 
-				"creation_year($params{year}) and ($params{pages})",
-				# add presence dependencies to update
-				# year calendar's links to months when
-				# pages are added/removed
-				deptype => deptype("presence"))) {
-		my $mtime = $IkiWiki::pagectime{$p};
-		my @date  = localtime($mtime);
-		my $month = $date[4] + 1;
 
-		$post_months[$month]++;
-	}
-		
+	my $pagespec = $params{pages};
+	my $year     = $params{year};
+	my $month    = $params{month};
+	my $pmonth   = $params{pmonth};
+	my $nmonth   = $params{nmonth};
+	my $pyear    = $params{pyear};
+	my $nyear    = $params{nyear};
+
 	my $calendar="\n";
-	
-	my $pyear = $params{year}  - 1;
-	my $nyear = $params{year}  + 1;
 
-	my $thisyear = $now[5]+1900;
 	my $future_month = 0;
-	$future_month = $now[4]+1 if $params{year} == $thisyear;
+	$future_month = $now[4]+1 if ($year == $now[5]+1900);
 
 	my $archivebase = 'archives';
 	$archivebase = $config{archivebase} if defined $config{archivebase};
 	$archivebase = $params{archivebase} if defined $params{archivebase};
 
 	# calculate URL's for previous and next years
-	my ($url, $purl, $nurl)=("$params{year}",'','');
-	if (exists $pagesources{"$archivebase/$params{year}"}) {
+	my ($url, $purl, $nurl)=("$year",'','');
+	if (exists $cache{$pagespec}{"$year"}) {
 		$url = htmllink($params{page}, $params{destpage}, 
-			"$archivebase/$params{year}",
-			noimageinline => 1,
-			linktext => $params{year},
-			title => $params{year});
+			"$archivebase/$year",
+			linktext => "$year");
 	}
-	add_depends($params{page}, "$archivebase/$params{year}", deptype("presence"));
-	if (exists $pagesources{"$archivebase/$pyear"}) {
+	add_depends($params{page}, "$archivebase/$year");
+	if (exists $cache{$pagespec}{"$pyear"}) {
 		$purl = htmllink($params{page}, $params{destpage}, 
 			"$archivebase/$pyear",
-			noimageinline => 1,
-			linktext => "\&larr;",
-			title => $pyear);
+			linktext => "\&larr;");
 	}
-	add_depends($params{page}, "$archivebase/$pyear", deptype("presence"));
-	if (exists $pagesources{"$archivebase/$nyear"}) {
+	add_depends($params{page}, "$archivebase/$pyear");
+	if (exists $cache{$pagespec}{"$nyear"}) {
 		$nurl = htmllink($params{page}, $params{destpage}, 
 			"$archivebase/$nyear",
-			noimageinline => 1,
-			linktext => "\&rarr;",
-			title => $nyear);
+			linktext => "\&rarr;");
 	}
-	add_depends($params{page}, "$archivebase/$nyear", deptype("presence"));
+	add_depends($params{page}, "$archivebase/$nyear");
 
 	# Start producing the year calendar
-	my $m=$params{months_per_row}-2;
 	$calendar=<<EOF;
 <table class="year-calendar">
-	<tr>
-	<th class="year-calendar-arrow">$purl</th>
- 	<th class="year-calendar-head" colspan="$m">$url</th>
-	<th class="year-calendar-arrow">$nurl</th>
-	</tr>
+	<caption class="year-calendar-head">
+	$purl
+	$url
+	$nurl
+	</caption>
 	<tr>
 		<th class="year-calendar-subhead" colspan="$params{months_per_row}">Months</th>
 	</tr>
 EOF
 
-	for (my $month = 1; $month <= 12; $month++) {
-		my @day=localtime(timelocal(0,0,0,15,$month-1,$params{year}-1900));
+	for ($month = 1; $month <= 12; $month++) {
+		my @day=localtime(timelocal(0,0,0,15,$month-1,$year-1900));
 		my $murl;
 		my $monthname = POSIX::strftime("%B", @day);
 		my $monthabbr = POSIX::strftime("%b", @day);
 		$calendar.=qq{\t<tr>\n}  if ($month % $params{months_per_row} == 1);
 		my $tag;
 		my $mtag=sprintf("%02d", $month);
-		if ($month == $params{month} && $thisyear == $params{year}) {
-			$tag = 'year-calendar-this-month';
+		if ($month == $params{month}) {
+			if ($cache{$pagespec}{"$year/$mtag"}) {
+				$tag = 'this_month_link';
+			}
+			else {
+				$tag = 'this_month_nolink';
+			}
 		}
-		elsif ($pagesources{"$archivebase/$params{year}/$mtag"}) {
-			$tag = 'year-calendar-month-link';
+		elsif ($cache{$pagespec}{"$year/$mtag"}) {
+			$tag = 'month_link';
 		} 
 		elsif ($future_month && $month >= $future_month) {
-			$tag = 'year-calendar-month-future';
+			$tag = 'month_future';
 		} 
 		else {
-			$tag = 'year-calendar-month-nolink';
+			$tag = 'month_nolink';
 		}
 
-		if ($pagesources{"$archivebase/$params{year}/$mtag"} &&
-		    $post_months[$mtag]) {
+		if ($cache{$pagespec}{"$year/$mtag"}) {
 			$murl = htmllink($params{page}, $params{destpage}, 
-				"$archivebase/$params{year}/$mtag",
-				noimageinline => 1,
-				linktext => $monthabbr,
-				title => $monthname);
+				"$archivebase/$year/$mtag",
+				linktext => "$monthabbr");
 			$calendar.=qq{\t<td class="$tag">};
 			$calendar.=$murl;
 			$calendar.=qq{\t</td>\n};
@@ -361,8 +310,7 @@ EOF
 		else {
 			$calendar.=qq{\t<td class="$tag">$monthabbr</td>\n};
 		}
-		add_depends($params{page}, "$archivebase/$params{year}/$mtag",
-			deptype("presence"));
+		add_depends($params{page}, "$archivebase/$year/$mtag");
 
 		$calendar.=qq{\t</tr>\n} if ($month % $params{months_per_row} == 0);
 	}
@@ -374,99 +322,76 @@ EOF
 	return $calendar;
 }
 
-sub setnextchange ($$) {
-	my $page=shift;
-	my $timestamp=shift;
-
-	if (! exists $pagestate{$page}{calendar}{nextchange} ||
-	    $pagestate{$page}{calendar}{nextchange} > $timestamp) {
-		$pagestate{$page}{calendar}{nextchange}=$timestamp;
-	}
-}
-
 sub preprocess (@) {
 	my %params=@_;
-
-	my $thisyear=1900 + $now[5];
-	my $thismonth=1 + $now[4];
-
 	$params{pages} = "*"            unless defined $params{pages};
 	$params{type}  = "month"        unless defined $params{type};
+	$params{month} = sprintf("%02d", $params{month}) if defined  $params{month};
 	$params{week_start_day} = 0     unless defined $params{week_start_day};
 	$params{months_per_row} = 3     unless defined $params{months_per_row};
-	$params{year}  = $thisyear	unless defined $params{year};
-	$params{month} = $thismonth	unless defined $params{month};
 
-	my $relativeyear=0;
-	if ($params{year} < 1) {
-		$relativeyear=1;
-		$params{year}=$thisyear+$params{year};
-	}
-	my $relativemonth=0;
-	if ($params{month} < 1) {
-		$relativemonth=1;
-		my $monthoff=$params{month};
-		$params{month}=($thismonth+$monthoff) % 12;
-		$params{month}=12 if $params{month}==0;
-		my $yearoff=POSIX::ceil(($thismonth-$params{month}) / -12)
-			- int($monthoff / 12);
-		$params{year}-=$yearoff;
-	}
-	
-	$params{month} = sprintf("%02d", $params{month});
-	
-	if ($params{type} eq 'month' && $params{year} == $thisyear
-	    && $params{month} == $thismonth) {
-		# calendar for current month, updates next midnight
-		setnextchange($params{destpage}, ($time
+	if (! defined $params{year} || ! defined $params{month}) {
+		# Record that the calendar next changes at midnight.
+		$pagestate{$params{destpage}}{calendar}{nextchange}=($time
 			+ (60 - $now[0])		# seconds
 			+ (59 - $now[1]) * 60		# minutes
 			+ (23 - $now[2]) * 60 * 60	# hours
-		));
-	}
-	elsif ($params{type} eq 'month' &&
-	       (($params{year} == $thisyear && $params{month} > $thismonth) ||
-	        $params{year} > $thisyear)) {
-		# calendar for upcoming month, updates 1st of that month
-		setnextchange($params{destpage},
-			timelocal(0, 0, 0, 1, $params{month}-1, $params{year}));
-	}
-	elsif (($params{type} eq 'year' && $params{year} == $thisyear) ||
-	       $relativemonth) {
-		# Calendar for current year updates 1st of next month.
-		# Any calendar relative to the current month also updates
-		# then.
-		if ($thismonth < 12) {
-			setnextchange($params{destpage},
-				timelocal(0, 0, 0, 1, $thismonth+1-1, $params{year}));
-		}
-		else {
-			setnextchange($params{destpage},
-				timelocal(0, 0, 0, 1, 1-1, $params{year}+1));
-		}
-	}
-	elsif ($relativeyear) {
-		# Any calendar relative to the current year updates 1st
-		# of next year.
-		setnextchange($params{destpage},
-			timelocal(0, 0, 0, 1, 1-1, $thisyear+1));
-	}
-	elsif ($params{type} eq 'year' && $params{year} > $thisyear) {
-		# calendar for upcoming year, updates 1st of that year
-		setnextchange($params{destpage},
-			timelocal(0, 0, 0, 1, 1-1, $params{year}));
+		);
+		
+		$params{year}  = 1900 + $now[5] unless defined $params{year};
+		$params{month} = 1    + $now[4] unless defined $params{month};
 	}
 	else {
-		# calendar for past month or year, does not need
-		# to update any more
 		delete $pagestate{$params{destpage}}{calendar};
 	}
 
-	my $calendar="";
-	if ($params{type} eq 'month') {
+	# Calculate month names for next month, and previous months
+	my $pmonth = $params{month} - 1;
+	my $nmonth = $params{month} + 1;
+	my $pyear  = $params{year}  - 1;
+	my $nyear  = $params{year}  + 1;
+
+	# Adjust for January and December
+	if ($params{month} == 1) {
+		$pmonth = 12;
+		$pyear--;
+	}
+	if ($params{month} == 12) {
+		$nmonth = 1;
+		$nyear++;
+	}
+
+	$params{pmonth}=$pmonth;
+	$params{nmonth}=$nmonth;
+	$params{pyear} =$pyear;
+	$params{nyear} =$nyear;
+
+	my $calendar="\n";
+	my $pagespec=$params{pages};
+	my $page =$params{page};
+
+	if (! defined $cache{$pagespec}) {
+		foreach my $p (pagespec_match_list([keys %pagesources], $pagespec)) {
+			my $mtime = $IkiWiki::pagectime{$p};
+			my $src   = $pagesources{$p};
+			my @date  = localtime($mtime);
+			my $mday  = $date[3];
+			my $month = $date[4] + 1;
+			my $year  = $date[5] + 1900;
+			my $mtag  = sprintf("%02d", $month);
+
+			# Only one posting per day is being linked to.
+			$linkcache{"$year/$mtag/$mday"} = "$src";
+			$cache{$pagespec}{"$year"}++;
+			$cache{$pagespec}{"$year/$mtag"}++;
+			$cache{$pagespec}{"$year/$mtag/$mday"}++;
+		}
+	}
+
+	if ($params{type} =~ /month/i) {
 		$calendar=format_month(%params);
 	}
-	elsif ($params{type} eq 'year') {
+	elsif ($params{type} =~ /year/i) {
 		$calendar=format_year(%params);
 	}
 
