@@ -9,6 +9,7 @@ use warnings;
 use strict;
 use IkiWiki 3.00;
 use Encode;
+use POSIX qw(strftime);
 
 use constant PREVIEW => "Preview";
 use constant POST_COMMENT => "Post comment";
@@ -20,22 +21,16 @@ my %commentstate;
 sub import {
 	hook(type => "checkconfig", id => 'comments',  call => \&checkconfig);
 	hook(type => "getsetup", id => 'comments',  call => \&getsetup);
-	hook(type => "preprocess", id => 'comment', call => \&preprocess,
-		scan => 1);
-	hook(type => "preprocess", id => 'commentmoderation', call => \&preprocess_moderation);
+	hook(type => "preprocess", id => 'comment', call => \&preprocess);
 	# here for backwards compatability with old comments
 	hook(type => "preprocess", id => '_comment', call => \&preprocess);
 	hook(type => "sessioncgi", id => 'comment', call => \&sessioncgi);
 	hook(type => "htmlize", id => "_comment", call => \&htmlize);
-	hook(type => "htmlize", id => "_comment_pending",
-		call => \&htmlize_pending);
 	hook(type => "pagetemplate", id => "comments", call => \&pagetemplate);
-	hook(type => "formbuilder_setup", id => "comments",
-		call => \&formbuilder_setup);
+	hook(type => "formbuilder_setup", id => "comments", call => \&formbuilder_setup);
 	# Load goto to fix up user page links for logged-in commenters
 	IkiWiki::loadplugin("goto");
 	IkiWiki::loadplugin("inline");
-	IkiWiki::loadplugin("transient");
 }
 
 sub getsetup () {
@@ -43,7 +38,6 @@ sub getsetup () {
 		plugin => {
 			safe => 1,
 			rebuild => 1,
-			section => "web",
 		},
 		comments_pagespec => {
 			type => 'pagespec',
@@ -91,31 +85,17 @@ sub getsetup () {
 			safe => 0,
 			rebuild => 0,
 		},
-		comments_allowformats => {
-			type => 'string',
-			default => '',
-			example => 'mdwn txt',
-			description => 'Restrict formats for comments to (no restriction if empty)',
-			safe => 1,
-			rebuild => 0,
-		},
-
 }
 
 sub checkconfig () {
 	$config{comments_commit} = 1
 		unless defined $config{comments_commit};
-	if (! $config{comments_commit}) {
-		$config{only_committed_changes}=0;
-	}
 	$config{comments_pagespec} = ''
 		unless defined $config{comments_pagespec};
 	$config{comments_closed_pagespec} = ''
 		unless defined $config{comments_closed_pagespec};
 	$config{comments_pagename} = 'comment_'
 		unless defined $config{comments_pagename};
-	$config{comments_allowformats} = ''
-		unless defined $config{comments_allowformats};
 }
 
 sub htmlize {
@@ -123,14 +103,6 @@ sub htmlize {
 	return $params{content};
 }
 
-sub htmlize_pending {
-	my %params = @_;
-	return sprintf(gettext("this comment needs %s"),
-		'<a href="'.
-		IkiWiki::cgiurl(do => "commentmoderation").'">'.
-		gettext("moderation").'</a>');
-}
-
 # FIXME: copied verbatim from meta
 sub safeurl ($) {
 	my $url=shift;
@@ -143,18 +115,12 @@ sub safeurl ($) {
 	}
 }
 
-sub isallowed ($) {
-    my $format = shift;
-    return ! $config{comments_allowformats} || $config{comments_allowformats} =~ /\b$format\b/;
-}
-
 sub preprocess {
 	my %params = @_;
 	my $page = $params{page};
 
 	my $format = $params{format};
-	if (defined $format && (! exists $IkiWiki::hooks{htmlize}{$format} ||
-				! isallowed($format))) {
+	if (defined $format && ! exists $IkiWiki::hooks{htmlize}{$format}) {
 		error(sprintf(gettext("unsupported page format %s"), $format));
 	}
 
@@ -164,27 +130,24 @@ sub preprocess {
 	}
 	$content =~ s/\\"/"/g;
 
-	if (defined wantarray) {
-		if ($config{comments_allowdirectives}) {
-			$content = IkiWiki::preprocess($page, $params{destpage},
-				$content);
-		}
+	$content = IkiWiki::filter($page, $params{destpage}, $content);
 
-		# no need to bother with htmlize if it's just HTML
-		$content = IkiWiki::htmlize($page, $params{destpage}, $format, $content)
-			if defined $format;
+	if ($config{comments_allowdirectives}) {
+		$content = IkiWiki::preprocess($page, $params{destpage},
+			$content);
+	}
 
-		IkiWiki::run_hooks(sanitize => sub {
-			$content = shift->(
-				page => $page,
-				destpage => $params{destpage},
-				content => $content,
-			);
-		});
-	}
-	else {
-		IkiWiki::preprocess($page, $params{destpage}, $content, 1);
-	}
+	# no need to bother with htmlize if it's just HTML
+	$content = IkiWiki::htmlize($page, $params{destpage}, $format, $content)
+		if defined $format;
+
+	IkiWiki::run_hooks(sanitize => sub {
+		$content = shift->(
+			page => $page,
+			destpage => $params{destpage},
+			content => $content,
+		);
+	});
 
 	# set metadata, possibly overriding [[!meta]] directives from the
 	# comment itself
@@ -202,14 +165,15 @@ sub preprocess {
 		if (defined $oiduser) {
 			# looks like an OpenID
 			$commentauthorurl = $commentuser;
-			$commentauthor = (defined $params{nickname} && length $params{nickname}) ? $params{nickname} : $oiduser;
+			$commentauthor = $oiduser;
 			$commentopenid = $commentuser;
 		}
 		else {
 			$commentauthorurl = IkiWiki::cgiurl(
 				do => 'goto',
-				page => IkiWiki::userpage($commentuser)
-			);
+				page => (length $config{userdir}
+					? "$config{userdir}/$commentuser"
+					: "$commentuser"));
 
 			$commentauthor = $commentuser;
 		}
@@ -226,7 +190,6 @@ sub preprocess {
 	$commentstate{$page}{commentip} = $commentip;
 	$commentstate{$page}{commentauthor} = $commentauthor;
 	$commentstate{$page}{commentauthorurl} = $commentauthorurl;
-	$commentstate{$page}{commentauthoravatar} = $params{avatar};
 	if (! defined $pagestate{$page}{meta}{author}) {
 		$pagestate{$page}{meta}{author} = $commentauthor;
 	}
@@ -243,7 +206,7 @@ sub preprocess {
 			my $url=$params{url};
 
 			eval q{use URI::Heuristic}; 
-			if (! $@) {
+		  	if (! $@) {
 				$url=URI::Heuristic::uf_uristr($url);
 			}
 
@@ -258,13 +221,11 @@ sub preprocess {
 	}
 
 	if (defined $params{subject}) {
-		# decode title the same way meta does
-		eval q{use HTML::Entities};
-		$pagestate{$page}{meta}{title} = decode_entities($params{subject});
+		$pagestate{$page}{meta}{title} = $params{subject};
 	}
 
-	if ($params{page} =~ m/\/\Q$config{comments_pagename}\E\d+_/) {
-		$pagestate{$page}{meta}{permalink} = urlto(IkiWiki::dirname($params{page})).
+	if ($params{page} =~ m/\/(\Q$config{comments_pagename}\E\d+)$/) {
+		$pagestate{$page}{meta}{permalink} = urlto(IkiWiki::dirname($params{page}), undef, 1).
 			"#".page_to_id($params{page});
 	}
 
@@ -277,22 +238,6 @@ sub preprocess {
 	return $content;
 }
 
-sub preprocess_moderation {
-	my %params = @_;
-
-	$params{desc}=gettext("Comment Moderation")
-		unless defined $params{desc};
-
-	if (length $config{cgiurl}) {
-		return '<a href="'.
-			IkiWiki::cgiurl(do => 'commentmoderation').
-			'">'.$params{desc}.'</a>';
-	}
-	else {
-		return $params{desc};
-	}
-}
-
 sub sessioncgi ($$) {
 	my $cgi=shift;
 	my $session=shift;
@@ -304,10 +249,6 @@ sub sessioncgi ($$) {
 	elsif ($do eq 'commentmoderation') {
 		commentmoderation($cgi, $session);
 	}
-	elsif ($do eq 'commentsignin') {
-		IkiWiki::cgi_signin($cgi, $session);
-		exit;
-	}
 }
 
 # Mostly cargo-culted from IkiWiki::plugin::editpage
@@ -322,17 +263,16 @@ sub editcomment ($$) {
 
 	my @buttons = (POST_COMMENT, PREVIEW, CANCEL);
 	my $form = CGI::FormBuilder->new(
-		fields => [qw{do sid page subject editcontent type author
-			email url subscribe anonsubscribe}],
+		fields => [qw{do sid page subject editcontent type author url}],
 		charset => 'utf-8',
 		method => 'POST',
 		required => [qw{editcontent}],
 		javascript => 0,
 		params => $cgi,
-		action => IkiWiki::cgiurl(),
+		action => $config{cgiurl},
 		header => 0,
 		table => 0,
-		template => { template('editcomment.tmpl') },
+		template => scalar IkiWiki::template_params('editcomment.tmpl'),
 	);
 
 	IkiWiki::decode_form_utf8($form);
@@ -353,7 +293,7 @@ sub editcomment ($$) {
 
 	my @page_types;
 	if (exists $IkiWiki::hooks{htmlize}) {
-		foreach my $key (grep { !/^_/ && isallowed($_) } keys %{$IkiWiki::hooks{htmlize}}) {
+		foreach my $key (grep { !/^_/ } keys %{$IkiWiki::hooks{htmlize}}) {
 			push @page_types, [$key, $IkiWiki::hooks{htmlize}{$key}{longname} || $key];
 		}
 	}
@@ -368,57 +308,42 @@ sub editcomment ($$) {
 	$form->field(name => "type", value => $type, force => 1,
 		type => 'select', options => \@page_types);
 
-	my $username=$session->param('name');
-	$form->tmpl_param(username => $username);
-		
-	$form->field(name => "subscribe", type => 'hidden');
-	$form->field(name => "anonsubscribe", type => 'hidden');
-	if (IkiWiki::Plugin::notifyemail->can("subscribe")) {
-		if (defined $username) {
-			$form->field(name => "subscribe", type => "checkbox",
-				options => [gettext("email replies to me")]);
-		}
-		elsif (IkiWiki::Plugin::passwordauth->can("anonuser")) {
-			$form->field(name => "anonsubscribe", type => "checkbox",
-				options => [gettext("email replies to me")]);
-		}
-	}
+	$form->tmpl_param(username => $session->param('name'));
 
 	if ($config{comments_allowauthor} and
 	    ! defined $session->param('name')) {
 		$form->tmpl_param(allowauthor => 1);
 		$form->field(name => 'author', type => 'text', size => '40');
-		$form->field(name => 'email', type => 'text', size => '40');
 		$form->field(name => 'url', type => 'text', size => '40');
 	}
 	else {
 		$form->tmpl_param(allowauthor => 0);
 		$form->field(name => 'author', type => 'hidden', value => '',
 			force => 1);
-		$form->field(name => 'email', type => 'hidden', value => '',
-			force => 1);
 		$form->field(name => 'url', type => 'hidden', value => '',
 			force => 1);
 	}
 
 	if (! defined $session->param('name')) {
 		# Make signinurl work and return here.
-		$form->tmpl_param(signinurl => IkiWiki::cgiurl(do => 'commentsignin'));
+		$form->tmpl_param(signinurl => IkiWiki::cgiurl(do => 'signin'));
 		$session->param(postsignin => $ENV{QUERY_STRING});
 		IkiWiki::cgi_savesession($session);
 	}
 
 	# The untaint is OK (as in editpage) because we're about to pass
-	# it to file_pruned and wiki_file_regexp anyway.
-	my ($page) = $form->field('page')=~/$config{wiki_file_regexp}/;
+	# it to file_pruned anyway
+	my $page = $form->field('page');
 	$page = IkiWiki::possibly_foolish_untaint($page);
 	if (! defined $page || ! length $page ||
-		IkiWiki::file_pruned($page)) {
+		IkiWiki::file_pruned($page, $config{srcdir})) {
 		error(gettext("bad page name"));
 	}
 
+	my $baseurl = urlto($page, undef, 1);
+
 	$form->title(sprintf(gettext("commenting on %s"),
-			IkiWiki::pagetitle(IkiWiki::basename($page))));
+			IkiWiki::pagetitle($page)));
 
 	$form->tmpl_param('helponformattinglink',
 		htmllink($page, $page, 'ikiwiki/formatting',
@@ -428,7 +353,8 @@ sub editcomment ($$) {
 
 	if ($form->submitted eq CANCEL) {
 		# bounce back to the page they wanted to comment on, and exit.
-		IkiWiki::redirect($cgi, urlto($page));
+		# CANCEL need not be considered in future
+		IkiWiki::redirect($cgi, urlto($page, undef, 1));
 		exit;
 	}
 
@@ -438,16 +364,6 @@ sub editcomment ($$) {
 			$page));
 	}
 
-	# There's no UI to get here, but someone might construct the URL,
-	# leading to a comment that exists in the repository but isn't
-	# shown
-	if (!pagespec_match($page, $config{comments_pagespec},
-		location => $page)) {
-		error(sprintf(gettext(
-			"comments on page '%s' are not allowed"),
-			$page));
-	}
-
 	if (pagespec_match($page, $config{comments_closed_pagespec},
 		location => $page)) {
 		error(sprintf(gettext(
@@ -461,23 +377,21 @@ sub editcomment ($$) {
 	IkiWiki::check_canedit($page, $cgi, $session);
 	$postcomment=0;
 
+	my $location=unique_comment_location($page, $config{srcdir});
+
 	my $content = "[[!comment format=$type\n";
 
+	# FIXME: handling of double quotes probably wrong?
 	if (defined $session->param('name')) {
 		my $username = $session->param('name');
 		$username =~ s/"/&quot;/g;
 		$content .= " username=\"$username\"\n";
 	}
-
-	if (defined $session->param('nickname')) {
-		my $nickname = $session->param('nickname');
-		$nickname =~ s/"/&quot;/g;
-		$content .= " nickname=\"$nickname\"\n";
-	}
-
-	if (!(defined $session->param('name') || defined $session->param('nickname')) &&
-		defined $session->remote_addr()) {
-		$content .= " ip=\"".$session->remote_addr()."\"\n";
+	elsif (defined $ENV{REMOTE_ADDR}) {
+		my $ip = $ENV{REMOTE_ADDR};
+		if ($ip =~ m/^([.0-9]+)$/) {
+			$content .= " ip=\"$1\"\n";
+		}
 	}
 
 	if ($config{comments_allowauthor}) {
@@ -493,32 +407,20 @@ sub editcomment ($$) {
 		}
 	}
 
-	my $avatar=getavatar($session->param('name'));
-	if (defined $avatar && length $avatar) {
-		$avatar =~ s/"/&quot;/g;
-		$content .= " avatar=\"$avatar\"\n";
-	}
-
 	my $subject = $form->field('subject');
 	if (defined $subject && length $subject) {
 		$subject =~ s/"/&quot;/g;
+		$content .= " subject=\"$subject\"\n";
 	}
-	else {
-		$subject = "comment ".(num_comments($page, $config{srcdir}) + 1);
-	}
-	$content .= " subject=\"$subject\"\n";
 
-	$content .= " date=\"" . strftime_utf8('%Y-%m-%dT%H:%M:%SZ', gmtime) . "\"\n";
+	$content .= " date=\"" . decode_utf8(strftime('%Y-%m-%dT%H:%M:%SZ', gmtime)) . "\"\n";
 
-	my $editcontent = $form->field('editcontent');
-	$editcontent="" if ! defined $editcontent;
+	my $editcontent = $form->field('editcontent') || '';
 	$editcontent =~ s/\r\n/\n/g;
 	$editcontent =~ s/\r/\n/g;
 	$editcontent =~ s/"/\\"/g;
 	$content .= " content=\"\"\"\n$editcontent\n\"\"\"]]\n";
 
-	my $location=unique_comment_location($page, $content, $config{srcdir});
-
 	# This is essentially a simplified version of editpage:
 	# - the user does not control the page that's created, only the parent
 	# - it's always a create operation, never an edit
@@ -539,20 +441,6 @@ sub editcomment ($$) {
 
 	if ($form->submitted eq POST_COMMENT && $form->validate) {
 		IkiWiki::checksessionexpiry($cgi, $session);
-
-		if (IkiWiki::Plugin::notifyemail->can("subscribe")) {
-			my $subspec="comment($page)";
-			if (defined $username &&
-			    length $form->field("subscribe")) {
-				IkiWiki::Plugin::notifyemail::subscribe(
-					$username, $subspec);
-			}
-			elsif (length $form->field("email") &&
-			       length $form->field("anonsubscribe")) {
-				IkiWiki::Plugin::notifyemail::anonsubscribe(
-					$form->field("email"), $subspec);
-			}
-		}
 		
 		$postcomment=1;
 		my $ok=IkiWiki::check_content(content => $form->field('editcontent'),
@@ -569,17 +457,11 @@ sub editcomment ($$) {
 		$postcomment=0;
 
 		if (! $ok) {
-			$location=unique_comment_location($page, $content, $IkiWiki::Plugin::transient::transientdir, "._comment_pending");
-			writefile("$location._comment_pending", $IkiWiki::Plugin::transient::transientdir, $content);
-
-			# Refresh so anything that deals with pending
-			# comments can be updated.
-			require IkiWiki::Render;
-			IkiWiki::refresh();
-			IkiWiki::saveindex();
-
+			my $penddir=$config{wikistatedir}."/comments_pending";
+			$location=unique_comment_location($page, $penddir);
+			writefile("$location._comment", $penddir, $content);
 			IkiWiki::printheader($session);
-			print IkiWiki::cgitemplate($cgi, gettext(gettext("comment stored for moderation")),
+			print IkiWiki::misctemplate(gettext(gettext("comment stored for moderation")),
 				"<p>".
 				gettext("Your comment will be posted after moderator review").
 				"</p>");
@@ -604,10 +486,8 @@ sub editcomment ($$) {
 
 			IkiWiki::rcs_add($file);
 			IkiWiki::disable_commit_hook();
-			$conflict = IkiWiki::rcs_commit_staged(
-				message => $message,
-				session => $session,
-			);
+			$conflict = IkiWiki::rcs_commit_staged($message,
+				$session->param('name'), $ENV{REMOTE_ADDR});
 			IkiWiki::enable_commit_hook();
 			IkiWiki::rcs_update();
 		}
@@ -624,44 +504,18 @@ sub editcomment ($$) {
 		# Jump to the new comment on the page.
 		# The trailing question mark tries to avoid broken
 		# caches and get the most recent version of the page.
-		IkiWiki::redirect($cgi, urlto($page).
+		IkiWiki::redirect($cgi, urlto($page, undef, 1).
 			"?updated#".page_to_id($location));
 
 	}
 	else {
-		IkiWiki::showform($form, \@buttons, $session, $cgi,
-			page => $page);
+		IkiWiki::showform ($form, \@buttons, $session, $cgi,
+			forcebaseurl => $baseurl);
 	}
 
 	exit;
 }
 
-sub getavatar ($) {
-	my $user=shift;
-	return undef unless defined $user;
-
-	my $avatar;
-	eval q{use Libravatar::URL};
-	if (! $@) {
-		my $oiduser = eval { IkiWiki::openiduser($user) };
-		my $https=defined $config{url} && $config{url}=~/^https:/;
-
-		if (defined $oiduser) {
-			eval {
-				$avatar = libravatar_url(openid => $user, https => $https);
-			}
-		}
-		if (! defined $avatar &&
-		    (my $email = IkiWiki::userinfo_get($user, 'email'))) {
-			eval {
-				$avatar = libravatar_url(email => $email, https => $https);
-			}
-		}
-	}
-	return $avatar;
-}
-
-
 sub commentmoderation ($$) {
 	my $cgi=shift;
 	my $session=shift;
@@ -681,37 +535,26 @@ sub commentmoderation ($$) {
 		my %vars=$cgi->Vars;
 		my $added=0;
 		foreach my $id (keys %vars) {
-			if ($id =~ /(.*)\._comment(?:_pending)?$/) {
-				$id=decode_utf8($id);
+			if ($id =~ /(.*)\Q._comment\E$/) {
 				my $action=$cgi->param($id);
 				next if $action eq 'Defer' && ! $rejectalldefer;
 
 				# Make sure that the id is of a legal
-				# pending comment.
-				my ($f) = $id =~ /$config{wiki_file_regexp}/;
+				# pending comment before untainting.
+				my ($f)= $id =~ /$config{wiki_file_regexp}/;
 				if (! defined $f || ! length $f ||
-				    IkiWiki::file_pruned($f)) {
+				    IkiWiki::file_pruned($f, $config{srcdir})) {
 					error("illegal file");
 				}
 
-				my $page=IkiWiki::dirname($f);
-				my $filedir=$IkiWiki::Plugin::transient::transientdir;
-				my $file="$filedir/$f";
-				if (! -e $file) {
-					# old location
-					$file="$config{srcdir}/$f";
-					$filedir=$config{srcdir};
-					if (! -e $file) {
-						# older location
-						$file="$config{wikistatedir}/comments_pending/".$f;
-						$filedir="$config{wikistatedir}/comments_pending";
-					}
-				}
+				my $page=IkiWiki::possibly_foolish_untaint(IkiWiki::dirname($1));
+				my $file="$config{wikistatedir}/comments_pending/".
+					IkiWiki::possibly_foolish_untaint($id);
 
 				if ($action eq 'Accept') {
 					my $content=eval { readfile($file) };
 					next if $@; # file vanished since form was displayed
-					my $dest=unique_comment_location($page, $content, $config{srcdir})."._comment";
+					my $dest=unique_comment_location($page, $config{srcdir})."._comment";
 					writefile($dest, $config{srcdir}, $content);
 					if ($config{rcs} and $config{comments_commit}) {
 						IkiWiki::rcs_add($dest);
@@ -719,8 +562,11 @@ sub commentmoderation ($$) {
 					$added++;
 				}
 
+				# This removes empty subdirs, so the
+				# .ikiwiki/comments_pending dir will
+				# go away when all are moderated.
 				require IkiWiki::Render;
-				IkiWiki::prune($file, $filedir);
+				IkiWiki::prune($file);
 			}
 		}
 
@@ -729,10 +575,8 @@ sub commentmoderation ($$) {
 			if ($config{rcs} and $config{comments_commit}) {
 				my $message = gettext("Comment moderation");
 				IkiWiki::disable_commit_hook();
-				$conflict=IkiWiki::rcs_commit_staged(
-					message => $message,
-					session => $session,
-				);
+				$conflict=IkiWiki::rcs_commit_staged($message,
+					$session->param('name'), $ENV{REMOTE_ADDR});
 				IkiWiki::enable_commit_hook();
 				IkiWiki::rcs_update();
 			}
@@ -747,28 +591,28 @@ sub commentmoderation ($$) {
 	}
 
 	my @comments=map {
-		my ($id, $dir, $ctime)=@{$_};
-		my $content=readfile("$dir/$id");
+		my ($id, $ctime)=@{$_};
+		my $file="$config{wikistatedir}/comments_pending/$id";
+		my $content=readfile($file);
 		my $preview=previewcomment($content, $id,
-			$id, $ctime);
+			IkiWiki::dirname($_), $ctime);
 		{
 			id => $id,
 			view => $preview,
-		}
-	} sort { $b->[2] <=> $a->[2] } comments_pending();
+		} 
+	} sort { $b->[1] <=> $a->[1] } comments_pending();
 
 	my $template=template("commentmoderation.tmpl");
 	$template->param(
 		sid => $session->id,
 		comments => \@comments,
-		cgiurl => IkiWiki::cgiurl(),
 	);
 	IkiWiki::printheader($session);
 	my $out=$template->output;
 	IkiWiki::run_hooks(format => sub {
 		$out = shift->(page => "", content => $out);
 	});
-	print IkiWiki::cgitemplate($cgi, gettext("comment moderation"), $out);
+	print IkiWiki::misctemplate(gettext("comment moderation"), $out);
 	exit;
 }
 
@@ -786,45 +630,30 @@ sub formbuilder_setup (@) {
 }
 
 sub comments_pending () {
-	my @ret;
+	my $dir="$config{wikistatedir}/comments_pending/";
+	return unless -d $dir;
 
+	my @ret;
 	eval q{use File::Find};
 	error($@) if $@;
-	eval q{use Cwd};
-	error($@) if $@;
-	my $origdir=getcwd();
-
-	my $find_comments=sub {
-		my $dir=shift;
-		my $extension=shift;
-		return unless -d $dir;
-
-		chdir($dir) || die "chdir $dir: $!";
-
-		find({
-			no_chdir => 1,
-			wanted => sub {
-				my $file=decode_utf8($_);
-				$file=~s/^\.\///;
-				return if ! length $file || IkiWiki::file_pruned($file)
-					|| -l $_ || -d _ || $file !~ /\Q$extension\E$/;
-				my ($f) = $file =~ /$config{wiki_file_regexp}/; # untaint
-				if (defined $f) {
-					my $ctime=(stat($_))[10];
-					push @ret, [$f, $dir, $ctime];
+	find({
+		no_chdir => 1,
+		wanted => sub {
+			$_=decode_utf8($_);
+			if (IkiWiki::file_pruned($_, $dir)) {
+				$File::Find::prune=1;
+			}
+			elsif (! -l $_ && ! -d _) {
+				$File::Find::prune=0;
+				my ($f)=/$config{wiki_file_regexp}/; # untaint
+				if (defined $f && $f =~ /\Q._comment\E$/) {
+					my $ctime=(stat($f))[10];
+					$f=~s/^\Q$dir\E\/?//;
+                                        push @ret, [$f, $ctime];
 				}
 			}
-		}, ".");
-
-		chdir($origdir) || die "chdir $origdir: $!";
-	};
-	
-	$find_comments->($IkiWiki::Plugin::transient::transientdir, "._comment_pending");
-	# old location
-	$find_comments->($config{srcdir}, "._comment_pending");
-	# old location
-	$find_comments->("$config{wikistatedir}/comments_pending/",
-		"._comment");
+		}
+	}, $dir);
 
 	return @ret;
 }
@@ -835,10 +664,6 @@ sub previewcomment ($$$) {
 	my $page=shift;
 	my $time=shift;
 
-	# Previewing a comment should implicitly enable comment posting mode.
-	my $oldpostcomment=$postcomment;
-	$postcomment=1;
-
 	my $preview = IkiWiki::htmlize($location, $page, '_comment',
 			IkiWiki::linkify($location, $page,
 			IkiWiki::preprocess($location, $page,
@@ -846,8 +671,7 @@ sub previewcomment ($$$) {
 
 	my $template = template("comment.tmpl");
 	$template->param(content => $preview);
-	$template->param(ctime => displaytime($time, undef, 1));
-	$template->param(html5 => $config{html5});
+	$template->param(ctime => displaytime($time));
 
 	IkiWiki::run_hooks(pagetemplate => sub {
 		shift->(page => $location,
@@ -857,16 +681,16 @@ sub previewcomment ($$$) {
 
 	$template->param(have_actions => 0);
 
-	$postcomment=$oldpostcomment;
-
 	return $template->output;
 }
 
 sub commentsshown ($) {
 	my $page=shift;
 
-	return pagespec_match($page, $config{comments_pagespec},
-		location => $page);
+	return ! pagespec_match($page, "internal(*/$config{comments_pagename}*)",
+	                        location => $page) &&
+	       pagespec_match($page, $config{comments_pagespec},
+	                      location => $page);
 }
 
 sub commentsopen ($) {
@@ -893,7 +717,7 @@ sub pagetemplate (@) {
 		my $comments = undef;
 		if ($shown) {
 			$comments = IkiWiki::preprocess_inline(
-				pages => "comment($page) and !comment($page/*)",
+				pages => "internal($page/$config{comments_pagename}*)",
 				template => 'comment',
 				show => 0,
 				reverse => 'yes',
@@ -909,43 +733,39 @@ sub pagetemplate (@) {
 		}
 
 		if ($shown && commentsopen($page)) {
-			$template->param(addcommenturl => addcommenturl($page));
+			my $addcommenturl = IkiWiki::cgiurl(do => 'comment',
+				page => $page);
+			$template->param(addcommenturl => $addcommenturl);
 		}
 	}
 
-	if ($shown) {
-		if ($template->query(name => 'commentsurl')) {
+	if ($template->query(name => 'commentsurl')) {
+		if ($shown) {
 			$template->param(commentsurl =>
-				urlto($page).'#comments');
+				urlto($page, undef, 1).'#comments');
 		}
+	}
 
-		if ($template->query(name => 'atomcommentsurl') && $config{usedirs}) {
+	if ($template->query(name => 'atomcommentsurl') && $config{usedirs}) {
+		if ($shown) {
 			# This will 404 until there are some comments, but I
 			# think that's probably OK...
 			$template->param(atomcommentsurl =>
-				urlto($page).'comments.atom');
+				urlto($page, undef, 1).'comments.atom');
 		}
+	}
 
-		if ($template->query(name => 'commentslink')) {
-			my $num=num_comments($page, $config{srcdir});
-			my $link;
-			if ($num > 0) {
-				$link = htmllink($page, $params{destpage}, $page,
-					linktext => sprintf(ngettext("%i comment", "%i comments", $num), $num),
+	if ($template->query(name => 'commentslink')) {
+		# XXX Would be nice to say how many comments there are in
+		# the link. But, to update the number, blog pages
+		# would have to update whenever comments of any inlines
+		# page are added, which is not currently done.
+		if ($shown) {
+			$template->param(commentslink =>
+				htmllink($page, $params{destpage}, $page,
+					linktext => gettext("Comments"),
 					anchor => "comments",
-					noimageinline => 1
-				);
-			}
-			elsif (commentsopen($page)) {
-				$link = "<a href=\"".addcommenturl($page)."\">".
-					#translators: Here "Comment" is a verb;
-					#translators: the user clicks on it to
-					#translators: post a comment.
-					gettext("Comment").
-					"</a>";
-			}
-			$template->param(commentslink => $link)
-				if defined $link;
+					noimageinline => 1));
 		}
 	}
 
@@ -984,11 +804,6 @@ sub pagetemplate (@) {
 			$commentstate{$page}{commentauthorurl});
 	}
 
-	if ($template->query(name => 'commentauthoravatar')) {
-		$template->param(commentauthoravatar =>
-			$commentstate{$page}{commentauthoravatar});
-	}
-
 	if ($template->query(name => 'removeurl') &&
 	    IkiWiki::Plugin::remove->can("check_canremove") &&
 	    length $config{cgiurl}) {
@@ -998,48 +813,30 @@ sub pagetemplate (@) {
 	}
 }
 
-sub addcommenturl ($) {
-	my $page=shift;
-
-	return IkiWiki::cgiurl(do => 'comment', page => $page);
-}
-
-sub num_comments ($$) {
+sub unique_comment_location ($) {
 	my $page=shift;
 	my $dir=shift;
 
-	my @comments=glob("$dir/$page/$config{comments_pagename}*._comment");
-	return int @comments;
-}
-
-sub unique_comment_location ($$$$) {
-	my $page=shift;
-	eval q{use Digest::MD5 'md5_hex'};
-	error($@) if $@;
-	my $content_md5=md5_hex(Encode::encode_utf8(shift));
-	my $dir=shift;
-	my $ext=shift || "._comment";
-
 	my $location;
-	my $i = num_comments($page, $dir);
+	my $i = 0;
 	do {
 		$i++;
-		$location = "$page/$config{comments_pagename}${i}_${content_md5}";
-	} while (-e "$dir/$location$ext");
+		$location = "$page/$config{comments_pagename}$i";
+	} while (-e "$dir/$location._comment");
 
 	return $location;
 }
 
 sub page_to_id ($) {
 	# Converts a comment page name into a unique, legal html id
-	# attribute value, that can be used as an anchor to link to the
+	# addtibute value, that can be used as an anchor to link to the
 	# comment.
 	my $page=shift;
 
 	eval q{use Digest::MD5 'md5_hex'};
 	error($@) if $@;
 
-	return "comment-".md5_hex(Encode::encode_utf8(($page)));
+	return "comment-".md5_hex($page);
 }
 	
 package IkiWiki::PageSpec;
@@ -1051,41 +848,7 @@ sub match_postcomment ($$;@) {
 	if (! $postcomment) {
 		return IkiWiki::FailReason->new("not posting a comment");
 	}
-	return match_glob($page, $glob, @_);
-}
-
-sub match_comment ($$;@) {
-	my $page = shift;
-	my $glob = shift;
-
-	if (! $postcomment) {
-		# To see if it's a comment, check the source file type.
-		# Deal with comments that were just deleted.
-		my $source=exists $IkiWiki::pagesources{$page} ?
-			$IkiWiki::pagesources{$page} :
-			$IkiWiki::delpagesources{$page};
-		my $type=defined $source ? IkiWiki::pagetype($source) : undef;
-		if (! defined $type || $type ne "_comment") {
-			return IkiWiki::FailReason->new("$page is not a comment");
-		}
-	}
-
-	return match_glob($page, "$glob/*", internal => 1, @_);
-}
-
-sub match_comment_pending ($$;@) {
-	my $page = shift;
-	my $glob = shift;
-	
-	my $source=exists $IkiWiki::pagesources{$page} ?
-		$IkiWiki::pagesources{$page} :
-		$IkiWiki::delpagesources{$page};
-	my $type=defined $source ? IkiWiki::pagetype($source) : undef;
-	if (! defined $type || $type ne "_comment_pending") {
-		return IkiWiki::FailReason->new("$page is not a pending comment");
-	}
-
-	return match_glob($page, "$glob/*", internal => 1, @_);
+	return match_glob($page, $glob);
 }
 
 1
