@@ -33,17 +33,12 @@ use warnings;
 use strict;
 use IkiWiki;
 
-use URI::Escape q{uri_escape_utf8};
 use File::chdir;
 
-
-# GENERAL PLUGIN API CALLS
-
 sub import {
+	hook(type => "genwrapper", id => "cvs", call => \&genwrapper);
 	hook(type => "checkconfig", id => "cvs", call => \&checkconfig);
 	hook(type => "getsetup", id => "cvs", call => \&getsetup);
-	hook(type => "genwrapper", id => "cvs", call => \&genwrapper);
-
 	hook(type => "rcs", id => "rcs_update", call => \&rcs_update);
 	hook(type => "rcs", id => "rcs_prepedit", call => \&rcs_prepedit);
 	hook(type => "rcs", id => "rcs_commit", call => \&rcs_commit);
@@ -54,7 +49,17 @@ sub import {
 	hook(type => "rcs", id => "rcs_recentchanges", call => \&rcs_recentchanges);
 	hook(type => "rcs", id => "rcs_diff", call => \&rcs_diff);
 	hook(type => "rcs", id => "rcs_getctime", call => \&rcs_getctime);
-	hook(type => "rcs", id => "rcs_getmtime", call => \&rcs_getmtime);
+}
+
+sub genwrapper () {
+	return <<EOF;
+	{
+		int j;
+		for (j = 1; j < argc; j++)
+			if (strstr(argv[j], "New directory") != NULL)
+				exit(0);
+	}
+EOF
 }
 
 sub checkconfig () {
@@ -80,7 +85,6 @@ sub getsetup () {
 		plugin => {
 			safe => 0, # rcs plugin
 			rebuild => undef,
-			section => "rcs",
 		},
 		cvsrepo => {
 			type => "string",
@@ -126,22 +130,39 @@ sub getsetup () {
 		},
 }
 
-sub genwrapper () {
-	return <<EOF;
-	{
-		int j;
-		for (j = 1; j < argc; j++)
-			if (strstr(argv[j], "New directory") != NULL)
-				exit(0);
-	}
-EOF
+sub cvs_info ($$) {
+	my $field=shift;
+	my $file=shift;
+
+	local $CWD = $config{srcdir};
+
+	my $info=`cvs status $file`;
+	my ($ret)=$info=~/^\s*$field:\s*(\S+)/m;
+	return $ret;
 }
 
+sub cvs_runcvs(@) {
+	my @cmd = @_;
+	unshift @cmd, 'cvs', '-Q';
 
-# VCS PLUGIN API CALLS
+	local $CWD = $config{srcdir};
+
+	open(my $savedout, ">&STDOUT");
+	open(STDOUT, ">", "/dev/null");
+	my $ret = system(@cmd);
+	open(STDOUT, ">&", $savedout);
+
+	return ($ret == 0) ? 1 : 0;
+}
+
+sub cvs_is_controlling {
+	my $dir=shift;
+	$dir=$config{srcdir} unless defined($dir);
+	return (-d "$dir/CVS") ? 1 : 0;
+}
 
 sub rcs_update () {
-	return unless cvs_is_controlling();
+	return unless cvs_is_controlling;
 	cvs_runcvs('update', '-dP');
 }
 
@@ -152,7 +173,7 @@ sub rcs_prepedit ($) {
 	# The file is relative to the srcdir.
 	my $file=shift;
 
-	return unless cvs_is_controlling();
+	return unless cvs_is_controlling;
 
 	# For cvs, return the revision of the file when
 	# editing begins.
@@ -160,29 +181,40 @@ sub rcs_prepedit ($) {
 	return defined $rev ? $rev : "";
 }
 
-sub rcs_commit (@) {
+sub rcs_commit ($$$;$$) {
 	# Tries to commit the page; returns undef on _success_ and
 	# a version of the page with the rcs's conflict markers on failure.
 	# The file is relative to the srcdir.
-	my %params=@_;
+	my $file=shift;
+	my $message=shift;
+	my $rcstoken=shift;
+	my $user=shift;
+	my $ipaddr=shift;
 
-	return unless cvs_is_controlling();
+	return unless cvs_is_controlling;
+
+	if (defined $user) {
+		$message="web commit by $user".(length $message ? ": $message" : "");
+	}
+	elsif (defined $ipaddr) {
+		$message="web commit from $ipaddr".(length $message ? ": $message" : "");
+	}
 
 	# Check to see if the page has been changed by someone
 	# else since rcs_prepedit was called.
-	my ($oldrev)=$params{token}=~/^([0-9]+)$/; # untaint
-	my $rev=cvs_info("Repository revision", "$config{srcdir}/$params{file}");
+	my ($oldrev)=$rcstoken=~/^([0-9]+)$/; # untaint
+	my $rev=cvs_info("Repository revision", "$config{srcdir}/$file");
 	if (defined $rev && defined $oldrev && $rev != $oldrev) {
 		# Merge their changes into the file that we've
 		# changed.
-		cvs_runcvs('update', $params{file}) ||
+		cvs_runcvs('update', $file) ||
 			warn("cvs merge from $oldrev to $rev failed\n");
 	}
 
 	if (! cvs_runcvs('commit', '-m',
-			 IkiWiki::possibly_foolish_untaint(commitmessage(%params)))) {
-		my $conflict=readfile("$config{srcdir}/$params{file}");
-		cvs_runcvs('update', '-C', $params{file}) ||
+			 IkiWiki::possibly_foolish_untaint $message)) {
+		my $conflict=readfile("$config{srcdir}/$file");
+		cvs_runcvs('update', '-C', $file) ||
 			warn("cvs revert failed\n");
 		return $conflict;
 	}
@@ -190,13 +222,20 @@ sub rcs_commit (@) {
 	return undef # success
 }
 
-sub rcs_commit_staged (@) {
+sub rcs_commit_staged ($$$) {
 	# Commits all staged changes. Changes can be staged using rcs_add,
 	# rcs_remove, and rcs_rename.
-	my %params=@_;
+	my ($message, $user, $ipaddr)=@_;
+
+	if (defined $user) {
+		$message="web commit by $user".(length $message ? ": $message" : "");
+	}
+	elsif (defined $ipaddr) {
+		$message="web commit from $ipaddr".(length $message ? ": $message" : "");
+	}
 
 	if (! cvs_runcvs('commit', '-m',
-			 IkiWiki::possibly_foolish_untaint(commitmessage(%params)))) {
+			 IkiWiki::possibly_foolish_untaint $message)) {
 		warn "cvs staged commit failed\n";
 		return 1; # failure
 	}
@@ -209,6 +248,9 @@ sub rcs_add ($) {
 	my $parent=IkiWiki::dirname($file);
 	my @files_to_add = ($file);
 
+	eval q{use File::MimeInfo};
+	error($@) if $@;
+
 	until ((length($parent) == 0) || cvs_is_controlling("$config{srcdir}/$parent")){
 		push @files_to_add, $parent;
 		$parent = IkiWiki::dirname($parent);
@@ -216,12 +258,21 @@ sub rcs_add ($) {
 
 	while ($file = pop @files_to_add) {
 		if (@files_to_add == 0) {
-			cvs_runcvs('add', cvs_keyword_subst_args($file)) ||
-				warn("cvs add file $file failed\n");
+			# file
+			my $filemime = File::MimeInfo::default($file);
+			if (defined($filemime) && $filemime eq 'text/plain') {
+				cvs_runcvs('add', $file) ||
+					warn("cvs add $file failed\n");
+			}
+			else {
+				cvs_runcvs('add', '-kb', $file) ||
+					warn("cvs add binary $file failed\n");
+			}
 		}
 		else {
+			# directory
 			cvs_runcvs('add', $file) ||
-				warn("cvs add dir $file failed\n");
+				warn("cvs add $file failed\n");
 		}
 	}
 }
@@ -230,7 +281,7 @@ sub rcs_remove ($) {
 	# filename is relative to the root of the srcdir
 	my $file=shift;
 
-	return unless cvs_is_controlling();
+	return unless cvs_is_controlling;
 
 	cvs_runcvs('rm', '-f', $file) ||
 		warn("cvs rm $file failed\n");
@@ -240,7 +291,7 @@ sub rcs_rename ($$) {
 	# filenames relative to the root of the srcdir
 	my ($src, $dest)=@_;
 
-	return unless cvs_is_controlling();
+	return unless cvs_is_controlling;
 
 	local $CWD = $config{srcdir};
 
@@ -252,11 +303,11 @@ sub rcs_rename ($$) {
 	rcs_remove($src);
 }
 
-sub rcs_recentchanges ($) {
+sub rcs_recentchanges($) {
 	my $num = shift;
 	my @ret;
 
-	return unless cvs_is_controlling();
+	return unless cvs_is_controlling;
 
 	eval q{use Date::Parse};
 	error($@) if $@;
@@ -314,10 +365,7 @@ sub rcs_recentchanges ($) {
 			$oldrev =~ s/INITIAL/0/;
 			$newrev =~ s/\(DEAD\)//;
 			my $diffurl = defined $config{diffurl} ? $config{diffurl} : "";
-			my $epage = join('/',
-				map { uri_escape_utf8($_) } split('/', $page)
-			);
-			$diffurl=~s/\[\[file\]\]/$epage/g;
+			$diffurl=~s/\[\[file\]\]/$page/g;
 			$diffurl=~s/\[\[r1\]\]/$oldrev/g;
 			$diffurl=~s/\[\[r2\]\]/$newrev/g;
 			unshift @pages, {
@@ -386,9 +434,8 @@ sub rcs_recentchanges ($) {
 	return @ret;
 }
 
-sub rcs_diff ($;$) {
+sub rcs_diff ($) {
 	my $rev=IkiWiki::possibly_foolish_untaint(int(shift));
-	my $maxlines=shift;
 
 	local $CWD = $config{srcdir};
 
@@ -396,15 +443,11 @@ sub rcs_diff ($;$) {
 	my @cvsps = `env TZ=UTC cvsps -q --cvs-direct -z 30 -g -s $rev`;
 	my $blank_lines_seen = 0;
 
-	# skip log, get to the diff
 	while (my $line = shift @cvsps) {
 		$blank_lines_seen++ if ($line =~ /^$/);
 		last if $blank_lines_seen == 2;
 	}
 
-	@cvsps = @cvsps[0..$maxlines-1]
-		if defined $maxlines && @cvsps > $maxlines;
-
 	if (wantarray) {
 		return @cvsps;
 	}
@@ -416,8 +459,6 @@ sub rcs_diff ($;$) {
 sub rcs_getctime ($) {
 	my $file=shift;
 
-	local $CWD = $config{srcdir};
-
 	my $cvs_log_infoline=qr/^date: (.+);\s+author/;
 
 	open CVSLOG, "cvs -Q log -r1.1 '$file' |"
@@ -443,107 +484,4 @@ sub rcs_getctime ($) {
 	return $date;
 }
 
-sub rcs_getmtime ($) {
-	error "rcs_getmtime is not implemented for cvs\n"; # TODO
-}
-
-
-# INTERNAL SUPPORT ROUTINES
-
-sub commitmessage (@) {
-	my %params=@_;
-
-	if (defined $params{session}) {
-		if (defined $params{session}->param("name")) {
-			return "web commit by ".
-				$params{session}->param("name").
-				(length $params{message} ? ": $params{message}" : "");
-		}
-		elsif (defined $params{session}->remote_addr()) {
-			return "web commit from ".
-				$params{session}->remote_addr().
-				(length $params{message} ? ": $params{message}" : "");
-		}
-	}
-	return $params{message};
-}
-
-sub cvs_info ($$) {
-	my $field=shift;
-	my $file=shift;
-
-	local $CWD = $config{srcdir};
-
-	my $info=`cvs status $file`;
-	my ($ret)=$info=~/^\s*$field:\s*(\S+)/m;
-	return $ret;
-}
-
-sub cvs_is_controlling {
-	my $dir=shift;
-	$dir=$config{srcdir} unless defined($dir);
-	return (-d "$dir/CVS") ? 1 : 0;
-}
-
-sub cvs_keyword_subst_args ($) {
-	my $file = shift;
-
-	local $CWD = $config{srcdir};
-
-	eval q{use File::MimeInfo};
-	error($@) if $@;
-	my $filemime = File::MimeInfo::default($file);
-	# if (-T $file) {
-
-	defined($filemime) && $filemime eq 'text/plain'
-		? return ('-kkv', $file)
-		: return ('-kb', $file);
-}
-
-sub cvs_runcvs(@) {
-	my @cmd = @_;
-	unshift @cmd, 'cvs', '-Q';
-
-	# CVS can't operate outside a srcdir, so we're always setting $CWD.
-	# "local $CWD" restores the previous value when we go out of scope.
-	# Usually that's correct. But if we're removing the last file from
-	# a directory, the post-commit hook will exec in a working directory
-	# that's about to not exist (CVS will prune it).
-	#
-	# chdir() manually here, so we can selectively not chdir() back.
-
-	my $oldcwd = $CWD;
-	chdir($config{srcdir});
-
-	eval q{
-		use IPC::Open3;
-		use Symbol qw(gensym);
-		use IO::File;
-	};
-	error($@) if $@;
-
-	my $cvsout = '';
-	my $cvserr = '';
-	local *CATCHERR = IO::File->new_tmpfile;
-	my $pid = open3(gensym(), \*CATCHOUT, ">&CATCHERR", @cmd);
-	while (my $l = <CATCHOUT>) {
-		$cvsout .= $l
-			unless 1;
-	}
-	waitpid($pid, 0);
-	my $ret = $? >> 8;
-	seek CATCHERR, 0, 0;
-	while (my $l = <CATCHERR>) {
-		$cvserr .= $l
-			unless $l =~ /^cvs commit: changing keyword expansion /;
-	}
-
-	print STDOUT $cvsout;
-	print STDERR $cvserr;
-
-	chdir($oldcwd) if -d $oldcwd;
-
-	return ($ret == 0) ? 1 : 0;
-}
-
 1
