@@ -18,7 +18,6 @@ sub import {
 	hook(type => "rcs", id => "rcs_recentchanges", call => \&rcs_recentchanges);
 	hook(type => "rcs", id => "rcs_diff", call => \&rcs_diff);
 	hook(type => "rcs", id => "rcs_getctime", call => \&rcs_getctime);
-	hook(type => "rcs", id => "rcs_getmtime", call => \&rcs_getmtime);
 }
 
 sub silentsystem (@) {
@@ -52,7 +51,7 @@ sub darcs_info ($$$) {
 	return $_;
 }
 
-sub file_in_vc ($$) {
+sub file_in_vc($$) {
 	my $repodir = shift;
 	my $file = shift;
 
@@ -63,23 +62,23 @@ sub file_in_vc ($$) {
 	}
 	my $found=0;
 	while (<DARCS_MANIFEST>) {
-		$found = 1 if /^(\.\/)?$file$/;
+		$found = 1, last if /^(\.\/)?$file$/;
 	}
 	close(DARCS_MANIFEST) or error("'darcs query manifest' exited " . $?);
 
 	return $found;
 }
 
-sub darcs_rev ($) {
+sub darcs_rev($) {
 	my $file = shift; # Relative to the repodir.
 	my $repodir = $config{srcdir};
 
-	return "" unless file_in_vc($repodir, $file);
+	return "" if (! file_in_vc($repodir, $file));
 	my $hash = darcs_info('hash', $repodir, $file);
 	return defined $hash ? $hash : "";
 }
 
-sub checkconfig () {
+sub checkconfig() {
 	if (defined $config{darcs_wrapper} && length $config{darcs_wrapper}) {
 		push @{$config{wrappers}}, {
 			wrapper => $config{darcs_wrapper},
@@ -88,12 +87,11 @@ sub checkconfig () {
 	}
 }
 
-sub getsetup () {
+sub getsetup() {
 	return
 		plugin => {
 			safe => 0, # rcs plugin
 			rebuild => undef,
-			section => "rcs",
 		},
 		darcs_wrapper => {
 			type => "string",
@@ -140,31 +138,14 @@ sub rcs_prepedit ($) {
 	return $rev;
 }
 
-sub commitauthor (@) {
-	my %params=@_;
-	
-	my $author="anon\@web";
-	if (defined $params{session}) {
-		if (defined $params{session}->param("name")) {
-			return $params{session}->param("name").'@web';
-		}
-		elsif (defined $params{session}->remote_addr()) {
-			return $params{session}->remote_addr().'@web';
-		}
-	}
-	return 'anon@web';
-}
-
-sub rcs_commit (@) {
+sub rcs_commit ($$$;$$) {
 	# Commit the page.  Returns 'undef' on success and a version of the page
 	# with conflict markers on failure.
-	my %params=@_;
 
-	my ($file, $message, $token) =
-		($params{file}, $params{message}, $params{token});
+	my ($file, $message, $rcstoken, $user, $ipaddr) = @_;
 
 	# Compute if the "revision" of $file changed.
-	my $changed = darcs_rev($file) ne $token;
+	my $changed = darcs_rev($file) ne $rcstoken;
 
 	# Yes, the following is a bit convoluted.
 	if ($changed) {
@@ -172,7 +153,7 @@ sub rcs_commit (@) {
 		rename("$config{srcdir}/$file", "$config{srcdir}/$file.save") or
 			error("failed to rename $file to $file.save: $!");
 
-		# Roll the repository back to $token.
+		# Roll the repository back to $rcstoken.
 
 		# TODO.  Can we be sure that no changes are lost?  I think that
 		# we can, if we make sure that the 'darcs push' below will always
@@ -183,28 +164,37 @@ sub rcs_commit (@) {
 		# TODO: 'yes | ...' needed?  Doesn't seem so.
 		silentsystem('darcs', "revert", "--repodir", $config{srcdir}, "--all") == 0 ||
 			error("'darcs revert' failed");
-		# Remove all patches starting at $token.
+		# Remove all patches starting at $rcstoken.
 		my $child = open(DARCS_OBLITERATE, "|-");
 		if (! $child) {
 			open(STDOUT, ">/dev/null");
 			exec('darcs', "obliterate", "--repodir", $config{srcdir},
-			   "--match", "hash " . $token) and
+			   "--match", "hash " . $rcstoken) and
 			   error("'darcs obliterate' failed");
 		}
 		1 while print DARCS_OBLITERATE "y";
 		close(DARCS_OBLITERATE);
-		# Restore the $token one.
+		# Restore the $rcstoken one.
 		silentsystem('darcs', "pull", "--quiet", "--repodir", $config{srcdir},
-			"--match", "hash " . $token, "--all") == 0 ||
+			"--match", "hash " . $rcstoken, "--all") == 0 ||
 			error("'darcs pull' failed");
 	
-		# We're back at $token.  Re-install the modified file.
+		# We're back at $rcstoken.  Re-install the modified file.
 		rename("$config{srcdir}/$file.save", "$config{srcdir}/$file") or
 			error("failed to rename $file.save to $file: $!");
 	}
 
 	# Record the changes.
-	my $author=commitauthor(%params);
+	my $author;
+	if (defined $user) {
+		$author = "$user\@web";
+	}
+	elsif (defined $ipaddr) {
+		$author = "$ipaddr\@web";
+	}
+	else {
+		$author = "anon\@web";
+	}
 	if (!defined $message || !length($message)) {
 		$message = "empty message";
 	}
@@ -219,13 +209,13 @@ sub rcs_commit (@) {
 
 	# If this updating yields any conflicts, we'll record them now to resolve
 	# them.  If nothing is recorded, there are no conflicts.
-	$token = darcs_rev($file);
+	$rcstoken = darcs_rev($file);
 	# TODO: Use only the first line here, i.e. only the patch name?
 	writefile("$file.log", $config{srcdir}, 'resolve conflicts: ' . $message);
 	silentsystem('darcs', 'record', '--repodir', $config{srcdir}, '--all',
 		'-m', 'resolve conflicts: ' . $message, '--author', $author, $file) == 0 ||
 		error("'darcs record' failed");
-	my $conflicts = darcs_rev($file) ne $token;
+	my $conflicts = darcs_rev($file) ne $rcstoken;
 	unlink("$config{srcdir}/$file.log") or
 		error("failed to remove '$file.log'");
 
@@ -247,18 +237,25 @@ sub rcs_commit (@) {
 	}
 }
 
-sub rcs_commit_staged (@) {
-	my %params=@_;
+sub rcs_commit_staged($$$) {
+	my ($message, $user, $ipaddr) = @_;
 
-	my $author=commitauthor(%params);
-	if (!defined $params{message} || !length($params{message})) {
-		$params{message} = "empty message";
+	my $author;
+	if (defined $user) {
+		$author = "$user\@web";
+	}
+	elsif (defined $ipaddr) {
+		$author = "$ipaddr\@web";
+	}
+	else {
+		$author = "anon\@web";
+	}
+	if (!defined $message || !length($message)) {
+		$message = "empty message";
 	}
 
-	silentsystem('darcs', "record", "--repodir", $config{srcdir},
-		"-a", "-A", $author,
-		"-m", $params{message},
-	) == 0 || error("'darcs record' failed");
+	silentsystem('darcs', "record", "--repodir", $config{srcdir}, "-a", "-A", $author,
+		"-m", $message)	== 0 || error("'darcs record' failed");
 
 	# Push the changes to the main repository.
 	silentsystem('darcs', 'push', '--quiet', '--repodir', $config{srcdir}, '--all') == 0 ||
@@ -396,11 +393,14 @@ sub rcs_getctime ($) {
 	eval q{use XML::Simple};
 	local $/=undef;
 
+	my $filer=substr($file, length($config{srcdir}));
+	$filer =~ s:^[/]+::;
+
 	my $child = open(LOG, "-|");
 	if (! $child) {
 		exec("darcs", "changes", "--xml", "--reverse",
-			"--repodir", $config{srcdir}, $file)
-		|| error("'darcs changes $file' failed to run");
+			"--repodir", $config{srcdir}, $filer)
+		|| error("'darcs changes $filer' failed to run");
 	}
 
 	my $data;
@@ -415,7 +415,7 @@ sub rcs_getctime ($) {
 	my $datestr = $log->{patch}[0]->{local_date};
 
 	if (! defined $datestr) {
-		warn "failed to get ctime for $file";
+		warn "failed to get ctime for $filer";
 		return 0;
 	}
 
@@ -426,8 +426,4 @@ sub rcs_getctime ($) {
 	return $date;
 }
 
-sub rcs_getmtime ($) {
-	error "rcs_getmtime is not implemented for darcs\n"; # TODO
-}
-
 1
