@@ -19,7 +19,6 @@ sub getsetup () {
 		plugin => {
 			safe => 1,
 			rebuild => undef,
-			section => "widget",
 		},
 }
 
@@ -27,10 +26,6 @@ sub preprocess (@) {
 	my ($image) = $_[0] =~ /$config{wiki_file_regexp}/; # untaint
 	my %params=@_;
 
-	if (! defined $image) {
-		error("bad image filename");
-	}
-
 	if (exists $imgdefaults{$params{page}}) {
 		foreach my $key (keys %{$imgdefaults{$params{page}}}) {
 			if (! exists $params{$key}) {
@@ -39,7 +34,7 @@ sub preprocess (@) {
 		}
 	}
 
-	if (! exists $params{size} || ! length $params{size}) {
+	if (! exists $params{size}) {
 		$params{size}='full';
 	}
 
@@ -49,7 +44,6 @@ sub preprocess (@) {
 	}
 
 	add_link($params{page}, $image);
-	add_depends($params{page}, $image);
 
 	# optimisation: detect scan mode, and avoid generating the image
 	if (! defined wantarray) {
@@ -64,146 +58,129 @@ sub preprocess (@) {
 
 	my $dir = $params{page};
 	my $base = IkiWiki::basename($file);
-	my $issvg = $base=~s/\.svg$/.png/i;
-	my $ispdf = $base=~s/\.pdf$/.png/i;
-	my $pagenumber = exists($params{pagenumber}) ? int($params{pagenumber}) : 0;
-	if ($pagenumber != 0) {
-		$base = "p$pagenumber-$base";
-	}
 
 	eval q{use Image::Magick};
 	error gettext("Image::Magick is not installed") if $@;
-	my $im = Image::Magick->new();
+	my $im = Image::Magick->new;
 	my $imglink;
-	my $imgdatalink;
-	my $r = $im->Read("$srcfile\[$pagenumber]");
-	error sprintf(gettext("failed to read %s: %s"), $file, $r) if $r;
-
-	if (! defined $im->Get("width") || ! defined $im->Get("height")) {
-		error sprintf(gettext("failed to get dimensions of %s"), $file);
-	}
+	my $r;
 
 	my ($dwidth, $dheight);
 
-	if ($params{size} eq 'full') {
-		$dwidth = $im->Get("width");
-		$dheight = $im->Get("height");
-	} else {
+	if ($params{size} ne 'full') {
+		add_depends($params{page}, $image);
+
 		my ($w, $h) = ($params{size} =~ /^(\d*)x(\d*)$/);
 		error sprintf(gettext('wrong size format "%s" (should be WxH)'), $params{size})
 			unless (defined $w && defined $h &&
 			        (length $w || length $h));
 
-		if ($im->Get("width") == 0 || $im->Get("height") == 0) {
-			($dwidth, $dheight)=(0, 0);
-		} elsif (! length $w || (length $h && $im->Get("height")*$w > $h * $im->Get("width"))) {
-			# using height because only height is given or ...
-			# because original image is more portrait than $w/$h
-			# ... slimness of $im > $h/w
-			# ... $im->Get("height")/$im->Get("width") > $h/$w
-			# ... $im->Get("height")*$w > $h * $im->Get("width")
-
-			$dheight=$h;
-			$dwidth=$h / $im->Get("height") * $im->Get("width");
-		} else { # (! length $h) or $w is what determines the resized size
-			$dwidth=$w;
-			$dheight=$w / $im->Get("width") * $im->Get("height");
-		}
-	}
-
-	if ($dwidth < $im->Get("width") || $ispdf) {
-		# resize down, or resize to pixels at all
-
-		my $outfile = "$config{destdir}/$dir/$params{size}-$base";
-		$imglink = "$dir/$params{size}-$base";
-
+		my $outfile = "$config{destdir}/$dir/${w}x${h}-$base";
+		$imglink = "$dir/${w}x${h}-$base";
+		
 		will_render($params{page}, $imglink);
 
 		if (-e $outfile && (-M $srcfile >= -M $outfile)) {
-			$im = Image::Magick->new;
 			$r = $im->Read($outfile);
 			error sprintf(gettext("failed to read %s: %s"), $outfile, $r) if $r;
 		}
 		else {
-			$r = $im->Resize(geometry => "${dwidth}x${dheight}");
+			$r = $im->Read($srcfile);
+			error sprintf(gettext("failed to read %s: %s"), $file, $r) if $r;
+
+			# don't resize any larger
+			my ($rw, $rh) = ($w, $h);
+			if ((length $rw && $rw > $im->Get("width")) ||
+			    (length $rh && $rh > $im->Get("height"))) {
+				$rw=$im->Get("width");
+				$rh=$im->Get("height");
+			}
+
+			$r = $im->Resize(geometry => "${rw}x${rh}");
 			error sprintf(gettext("failed to resize: %s"), $r) if $r;
 
-			$im->set(($issvg || $ispdf) ? (magick => 'png') : ());
-			my @blob = $im->ImageToBlob();
-			# don't actually write resized file in preview mode;
-			# rely on width and height settings
+			# don't actually write file in preview mode
 			if (! $params{preview}) {
+				my @blob = $im->ImageToBlob();
 				writefile($imglink, $config{destdir}, $blob[0], 1);
 			}
 			else {
-				eval q{use MIME::Base64};
-				error($@) if $@;
-				$imgdatalink = "data:image/".$im->Get("magick").";base64,".encode_base64($blob[0]);
+				$imglink = $file;
 			}
 		}
 
-		# always get the true size of the resized image (it could be
-		# that imagemagick did its calculations differently)
-		$dwidth  = $im->Get("width");
-		$dheight = $im->Get("height");
-	} else {
-		$imglink = $file;
+		# since we don't really resize larger, set the display
+		# size, so the browser can scale the image up if necessary
+		if (length $w && length $h) {
+			($dwidth, $dheight)=($w, $h);
+		}
+		# avoid division by zero on 0x0 image
+		elsif ($im->Get("width") == 0 || $im->Get("height") == 0) {
+			($dwidth, $dheight)=(0, 0);
+		}
+		# calculate unspecified size from the other one, preserving
+		# aspect ratio
+		elsif (length $w) {
+			$dwidth=$w;
+			$dheight=$w / $im->Get("width") * $im->Get("height");
+		}
+		elsif (length $h) {
+			$dheight=$h;
+			$dwidth=$h / $im->Get("height") * $im->Get("width");
+		}
+
 	}
-	
-	if (! defined($dwidth) || ! defined($dheight)) {
-		error sprintf(gettext("failed to determine size of image %s"), $file)
+	else {
+		$r = $im->Read($srcfile);
+		error sprintf(gettext("failed to read %s: %s"), $file, $r) if $r;
+		$imglink = $file;
+		$dwidth = $im->Get("width");
+		$dheight = $im->Get("height");
 	}
 
 	my ($fileurl, $imgurl);
-	my $urltobase = $params{preview} ? undef : $params{destpage};
-	$fileurl=urlto($file, $urltobase);
-	$imgurl=$imgdatalink ? $imgdatalink : urlto($imglink, $urltobase);
-
-	if (! exists $params{class}) {
-		$params{class}="img";
+	if (! $params{preview}) {
+		$fileurl=urlto($file, $params{destpage});
+		$imgurl=urlto($imglink, $params{destpage});
+	}
+	else {
+		$fileurl="$config{url}/$file";
+		$imgurl="$config{url}/$imglink";
 	}
 
-	my $attrs='';
-	foreach my $attr (qw{alt title class id hspace vspace}) {
-		if (exists $params{$attr}) {
-			$attrs.=" $attr=\"$params{$attr}\"";
-		}
+	if (! defined($im->Get("width")) || ! defined($im->Get("height"))) {
+		error sprintf(gettext("failed to determine size of image %s"), $file)
 	}
-	
+
 	my $imgtag='<img src="'.$imgurl.
 		'" width="'.$dwidth.
 		'" height="'.$dheight.'"'.
-		$attrs.
-		(exists $params{align} && ! exists $params{caption} ? ' align="'.$params{align}.'"' : '').
+		(exists $params{alt} ? ' alt="'.$params{alt}.'"' : '').
+		(exists $params{title} ? ' title="'.$params{title}.'"' : '').
+		(exists $params{align} ? ' align="'.$params{align}.'"' : '').
+		(exists $params{class} ? ' class="'.$params{class}.'"' : '').
+		(exists $params{id} ? ' id="'.$params{id}.'"' : '').
 		' />';
 
-	my $link;
-	if (! defined $params{link}) {
-		$link=$fileurl;
+	if (! defined $params{link} || lc($params{link}) eq 'yes') {
+		$imgtag='<a href="'.$fileurl.'">'.$imgtag.'</a>';
 	}
 	elsif ($params{link} =~ /^\w+:\/\//) {
-		$link=$params{link};
-	}
-
-	if (defined $link) {
-		$imgtag='<a href="'.$link.'">'.$imgtag.'</a>';
+		$imgtag='<a href="'.$params{link}.'">'.$imgtag.'</a>';
 	}
 	else {
 		my $b = bestlink($params{page}, $params{link});
 	
 		if (length $b) {
-			add_depends($params{page}, $b, deptype("presence"));
+			add_depends($params{page}, $b);
 			$imgtag=htmllink($params{page}, $params{destpage},
 				$params{link}, linktext => $imgtag,
-				noimageinline => 1,
-			);
+				noimageinline => 1);
 		}
 	}
 
 	if (exists $params{caption}) {
-		return '<table class="img'.
-			(exists $params{align} ? " align-$params{align}" : "").
-			'">'.
+		return '<table class="img">'.
 			'<caption>'.$params{caption}.'</caption>'.
 			'<tr><td>'.$imgtag.'</td></tr>'.
 			'</table>';
