@@ -19,14 +19,14 @@ sub import {
 	hook(type => "checkconfig", id => "inline", call => \&checkconfig);
 	hook(type => "sessioncgi", id => "inline", call => \&sessioncgi);
 	hook(type => "preprocess", id => "inline", 
-		call => \&IkiWiki::preprocess_inline, scan => 1);
+		call => \&IkiWiki::preprocess_inline);
 	hook(type => "pagetemplate", id => "inline",
 		call => \&IkiWiki::pagetemplate_inline);
 	hook(type => "format", id => "inline", call => \&format, first => 1);
 	# Hook to change to do pinging since it's called late.
 	# This ensures each page only pings once and prevents slow
 	# pings interrupting page builds.
-	hook(type => "rendered", id => "inline", call => \&IkiWiki::pingurl);
+	hook(type => "change", id => "inline", call => \&IkiWiki::pingurl);
 }
 
 sub getopt () {
@@ -49,7 +49,6 @@ sub getsetup () {
 		plugin => {
 			safe => 1,
 			rebuild => undef,
-			section => "core",
 		},
 		rss => {
 			type => "boolean",
@@ -104,7 +103,7 @@ sub checkconfig () {
 }
 
 sub format (@) {
-	my %params=@_;
+        my %params=@_;
 
 	# Fill in the inline content generated earlier. This is actually an
 	# optimisation.
@@ -119,7 +118,7 @@ sub sessioncgi ($$) {
 	my $session=shift;
 
 	if ($q->param('do') eq 'blog') {
-		my $page=titlepage(decode_utf8(scalar $q->param('title')));
+		my $page=titlepage(decode_utf8($q->param('title')));
 		$page=~s/(\/)/"__".ord($1)."__"/eg; # don't create subdirs
 		# if the page already exists, munge it to be unique
 		my $from=$q->param('from');
@@ -128,10 +127,10 @@ sub sessioncgi ($$) {
 			$add=1 unless length $add;
 			$add++;
 		}
-		$q->param('page', "/$from/$page$add");
+		$q->param('page', $page.$add);
 		# now go create the page
 		$q->param('do', 'create');
-		# make sure the editpage plugin is loaded
+		# make sure the editpage plugin in loaded
 		if (IkiWiki->can("cgi_editpage")) {
 			IkiWiki::cgi_editpage($q, $session);
 		}
@@ -155,29 +154,12 @@ sub preprocess_inline (@) {
 	if (! exists $params{pages} && ! exists $params{pagenames}) {
 		error gettext("missing pages parameter");
 	}
-
-	if (! defined wantarray) {
-		# Running in scan mode: only do the essentials
-
-		if (yesno($params{trail}) && IkiWiki::Plugin::trail->can("preprocess_trailitems")) {
-			# default to sorting age, the same as inline itself,
-			# but let the params override that
-			IkiWiki::Plugin::trail::preprocess_trailitems(sort => 'age', %params);
-		}
-
-		return;
-	}
-
-	if (yesno($params{trail}) && IkiWiki::Plugin::trail->can("preprocess_trailitems")) {
-		scalar IkiWiki::Plugin::trail::preprocess_trailitems(sort => 'age', %params);
-	}
-
 	my $raw=yesno($params{raw});
 	my $archive=yesno($params{archive});
 	my $rss=(($config{rss} || $config{allowrss}) && exists $params{rss}) ? yesno($params{rss}) : $config{rss};
 	my $atom=(($config{atom} || $config{allowatom}) && exists $params{atom}) ? yesno($params{atom}) : $config{atom};
 	my $quick=exists $params{quick} ? yesno($params{quick}) : 0;
-	my $feeds=exists $params{feeds} ? yesno($params{feeds}) : !$quick && ! $raw;
+	my $feeds=exists $params{feeds} ? yesno($params{feeds}) : !$quick;
 	my $emptyfeeds=exists $params{emptyfeeds} ? yesno($params{emptyfeeds}) : 1;
 	my $feedonly=yesno($params{feedonly});
 	if (! exists $params{show} && ! $archive) {
@@ -186,13 +168,6 @@ sub preprocess_inline (@) {
 	if (! exists $params{feedshow} && exists $params{show}) {
 		$params{feedshow}=$params{show};
 	}
-	my $title;
-	if (exists $params{title}) {
-		$title = $params{title};
-	}
-	else {
-		$title = $params{page} ne "index" ? pagetitle($params{page}) : $config{wikiname};
-	}
 	my $desc;
 	if (exists $params{description}) {
 		$desc = $params{description} 
@@ -218,41 +193,43 @@ sub preprocess_inline (@) {
 			}
 		}
 
-		@list = split ' ', $params{pagenames};
-
-		if (yesno($params{reverse})) {
-			@list=reverse(@list);
-		}
-
-		foreach my $p (@list) {
-			add_depends($params{page}, $p, deptype($quick ? "presence" : "content"));
-		}
-
-		@list = grep { exists $pagesources{$_} } @list;
+		@list = map { bestlink($params{page}, $_) }
+		        split ' ', $params{pagenames};
 	}
 	else {
-		my $num=0;
-		if ($params{show}) {
-			$num=$params{show};
-		}
-		if ($params{feedshow} && $num < $params{feedshow} && $num > 0) {
-			$num=$params{feedshow};
-		}
-		if ($params{skip} && $num) {
-			$num+=$params{skip};
-		}
+		add_depends($params{page}, $params{pages});
 
-		@list = pagespec_match_list($params{page}, $params{pages},
-			deptype => deptype($quick ? "presence" : "content"),
-			filter => sub { $_[0] eq $params{page} },
-			sort => exists $params{sort} ? $params{sort} : "age",
-			reverse => yesno($params{reverse}),
-			($num ? (num => $num) : ()),
-		);
+		@list = pagespec_match_list(
+			[ grep { $_ ne $params{page} } keys %pagesources ],
+			$params{pages}, location => $params{page});
+
+		if (exists $params{sort} && $params{sort} eq 'title') {
+			@list=sort { pagetitle(basename($a)) cmp pagetitle(basename($b)) } @list;
+		}
+		elsif (exists $params{sort} && $params{sort} eq 'title_natural') {
+			eval q{use Sort::Naturally};
+			if ($@) {
+				error(gettext("Sort::Naturally needed for title_natural sort"));
+			}
+			@list=sort { Sort::Naturally::ncmp(pagetitle(basename($a)), pagetitle(basename($b))) } @list;
+		}
+		elsif (exists $params{sort} && $params{sort} eq 'mtime') {
+			@list=sort { $pagemtime{$b} <=> $pagemtime{$a} } @list;
+		}
+		elsif (! exists $params{sort} || $params{sort} eq 'age') {
+			@list=sort { $pagectime{$b} <=> $pagectime{$a} } @list;
+		}
+		else {
+			error sprintf(gettext("unknown sort type %s"), $params{sort});
+		}
+	}
+
+	if (yesno($params{reverse})) {
+		@list=reverse(@list);
 	}
 
 	if (exists $params{skip}) {
-		@list=@list[$params{skip} .. $#list];
+		@list=@list[$params{skip} .. scalar @list - 1];
 	}
 	
 	my @feedlist;
@@ -270,12 +247,15 @@ sub preprocess_inline (@) {
 		@list=@list[0..$params{show} - 1];
 	}
 
+	# Explicitly add all currently displayed pages as dependencies, so
+	# that if they are removed or otherwise changed, the inline will be
+	# sure to be updated.
+	foreach my $p ($#list >= $#feedlist ? @list : @feedlist) {
+		add_depends($params{page}, $p);
+	}
+	
 	if ($feeds && exists $params{feedpages}) {
-		@feedlist = pagespec_match_list(
-			$params{page}, "($params{pages}) and ($params{feedpages})",
-			deptype => deptype($quick ? "presence" : "content"),
-			list => \@feedlist,
-		);
+		@feedlist=pagespec_match_list(\@feedlist, $params{feedpages}, location => $params{page});
 	}
 
 	my ($feedbase, $feednum);
@@ -294,7 +274,7 @@ sub preprocess_inline (@) {
 			}
 			$params{feedfile}=possibly_foolish_untaint($params{feedfile});
 		}
-		$feedbase=targetpage($params{page}, "", $params{feedfile});
+		$feedbase=targetpage($params{destpage}, "", $params{feedfile});
 
 		my $feedid=join("\0", $feedbase, map { $_."\0".$params{$_} } sort keys %params);
 		if (exists $knownfeeds{$feedid}) {
@@ -315,41 +295,20 @@ sub preprocess_inline (@) {
 		}
 	}
 
-	my ($rssurl, $atomurl, $rssdesc, $atomdesc);
-	if ($feeds) {
-		if ($rss) {
-			$rssurl=abs2rel($feedbase."rss".$feednum, dirname(htmlpage($params{destpage})));
-			$rssdesc = sprintf(gettext("%s (RSS feed)"), $desc);
-		}
-		if ($atom) {
-			$atomurl=abs2rel($feedbase."atom".$feednum, dirname(htmlpage($params{destpage})));
-			$atomdesc = sprintf(gettext("%s (Atom feed)"), $desc);
-		}
-	}
+	my $rssurl=abs2rel($feedbase."rss".$feednum, dirname(htmlpage($params{destpage}))) if $feeds && $rss;
+	my $atomurl=abs2rel($feedbase."atom".$feednum, dirname(htmlpage($params{destpage}))) if $feeds && $atom;
 
 	my $ret="";
 
-	my $postform = (exists $params{rootpage});
-	if (exists $params{postform}) {
-		$postform = yesno($params{postform});
-	}
-
-	if (length $config{cgiurl} && ! $params{preview} && $postform &&
+	if (length $config{cgiurl} && ! $params{preview} && (exists $params{rootpage} ||
+	    (exists $params{postform} && yesno($params{postform}))) &&
 	    IkiWiki->can("cgi_editpage")) {
 		# Add a blog post form, with feed buttons.
-		my $formtemplate=template_depends("blogpost.tmpl", $params{page}, blind_cache => 1);
-		$formtemplate->param(cgiurl => IkiWiki::cgiurl());
+		my $formtemplate=template("blogpost.tmpl", blind_cache => 1);
+		$formtemplate->param(cgiurl => $config{cgiurl});
 		$formtemplate->param(rootpage => rootpage(%params));
-		if ($feeds) {
-			if ($rss) {
-				$formtemplate->param(rssurl => $rssurl);
-				$formtemplate->param(rssdesc => $rssdesc);
-			}
-			if ($atom) {
-				$formtemplate->param(atomurl => $atomurl);
-				$formtemplate->param(atomdesc => $atomdesc);
-			}
-		}
+		$formtemplate->param(rssurl => $rssurl) if $feeds && $rss;
+		$formtemplate->param(atomurl => $atomurl) if $feeds && $atom;
 		if (exists $params{postformtext}) {
 			$formtemplate->param(postformtext =>
 				$params{postformtext});
@@ -358,10 +317,6 @@ sub preprocess_inline (@) {
 			$formtemplate->param(postformtext =>
 				gettext("Add a new post titled:"));
 		}
-		if (exists $params{id}) {
-			$formtemplate->param(postformid =>
-				$params{id});
-		}
 		$ret.=$formtemplate->output;
 	    	
 		# The post form includes the feed buttons, so
@@ -370,43 +325,25 @@ sub preprocess_inline (@) {
 	}
 	elsif ($feeds && !$params{preview} && ($emptyfeeds || @feedlist)) {
 		# Add feed buttons.
-		my $linktemplate=template_depends("feedlink.tmpl", $params{page}, blind_cache => 1);
-		if ($rss) {
-			$linktemplate->param(rssurl => $rssurl);
-			$linktemplate->param(rssdesc => $rssdesc);
-		}
-		if ($atom) {
-			$linktemplate->param(atomurl => $atomurl);
-			$linktemplate->param(atomdesc => $atomdesc);
-		}
-		if (exists $params{id}) {
-			$linktemplate->param(id => $params{id});
-		}
+		my $linktemplate=template("feedlink.tmpl", blind_cache => 1);
+		$linktemplate->param(rssurl => $rssurl) if $rss;
+		$linktemplate->param(atomurl => $atomurl) if $atom;
 		$ret.=$linktemplate->output;
 	}
 	
 	if (! $feedonly) {
-		my $template;
-		if (! $raw) {
-			# cannot use wiki pages as templates; template not sanitized due to
-			# format hook hack
-			eval {
-				$template=template_depends($params{template}.".tmpl", $params{page},
-					blind_cache => 1);
-			};
-			if ($@) {
-				# gettext can clobber $@
-				my $error = $@;
-				error sprintf(gettext("failed to process template %s"), $params{template}.".tmpl").": $error";
-			}
+		require HTML::Template;
+		my @params=IkiWiki::template_params($params{template}.".tmpl", blind_cache => 1);
+		if (! @params) {
+			error sprintf(gettext("nonexistant template %s"), $params{template});
 		}
-		my $needcontent=$raw || (!($archive && $quick) && $template->query(name => 'content'));
+		my $template=HTML::Template->new(@params) unless $raw;
 	
 		foreach my $page (@list) {
 			my $file = $pagesources{$page};
 			my $type = pagetype($file);
-			if (! $raw) {
-				if ($needcontent) {
+			if (! $raw || ($raw && ! defined $type)) {
+				unless ($archive && $quick) {
 					# Get the content before populating the
 					# template, since getting the content uses
 					# the same template if inlines are nested.
@@ -416,19 +353,18 @@ sub preprocess_inline (@) {
 				$template->param(pageurl => urlto($page, $params{destpage}));
 				$template->param(inlinepage => $page);
 				$template->param(title => pagetitle(basename($page)));
-				$template->param(ctime => displaytime($pagectime{$page}, $params{timeformat}, 1));
+				$template->param(ctime => displaytime($pagectime{$page}, $params{timeformat}));
 				$template->param(mtime => displaytime($pagemtime{$page}, $params{timeformat}));
 				$template->param(first => 1) if $page eq $list[0];
 				$template->param(last => 1) if $page eq $list[$#list];
-				$template->param(html5 => $config{html5});
 	
 				if ($actions) {
 					my $file = $pagesources{$page};
 					my $type = pagetype($file);
 					if ($config{discussion}) {
-						if ($page !~ /.*\/\Q$config{discussionpage}\E$/i &&
+						if ($page !~ /.*\/\Q$config{discussionpage}\E$/ &&
 						    (length $config{cgiurl} ||
-						     exists $pagesources{$page."/".lc($config{discussionpage})})) {
+						     exists $links{$page."/".$config{discussionpage}})) {
 							$template->param(have_actions => 1);
 							$template->param(discussionlink =>
 								htmllink($page,
@@ -438,12 +374,9 @@ sub preprocess_inline (@) {
 									forcesubpage => 1));
 						}
 					}
-					if (length $config{cgiurl} &&
-					    defined $type &&
-					    IkiWiki->can("cgi_editpage")) {
+					if (length $config{cgiurl} && defined $type) {
 						$template->param(have_actions => 1);
 						$template->param(editurl => cgiurl(do => "edit", page => $page));
-
 					}
 				}
 	
@@ -463,10 +396,6 @@ sub preprocess_inline (@) {
 					      filter($page, $params{destpage},
 					      readfile(srcfile($file)))));
 				}
-				else {
-					$ret.="\n".
-					      readfile(srcfile($file));
-				}
 			}
 		}
 	}
@@ -478,9 +407,9 @@ sub preprocess_inline (@) {
 			if (! $params{preview}) {
 				writefile($rssp, $config{destdir},
 					genfeed("rss",
-						$config{url}."/".$rssp, $title, $desc, $params{guid}, $params{page}, @feedlist));
+						$config{url}."/".$rssp, $desc, $params{guid}, $params{destpage}, @feedlist));
 				$toping{$params{destpage}}=1 unless $config{rebuild};
-				$feedlinks{$params{destpage}}.=qq{<link rel="alternate" type="application/rss+xml" title="$rssdesc" href="$rssurl" />};
+				$feedlinks{$params{destpage}}.=qq{<link rel="alternate" type="application/rss+xml" title="$desc (RSS)" href="$rssurl" />};
 			}
 		}
 		if ($atom) {
@@ -488,15 +417,13 @@ sub preprocess_inline (@) {
 			will_render($params{destpage}, $atomp);
 			if (! $params{preview}) {
 				writefile($atomp, $config{destdir},
-					genfeed("atom", $config{url}."/".$atomp, $title, $desc, $params{guid}, $params{page}, @feedlist));
+					genfeed("atom", $config{url}."/".$atomp, $desc, $params{guid}, $params{destpage}, @feedlist));
 				$toping{$params{destpage}}=1 unless $config{rebuild};
-				$feedlinks{$params{destpage}}.=qq{<link rel="alternate" type="application/atom+xml" title="$atomdesc" href="$atomurl" />};
+				$feedlinks{$params{destpage}}.=qq{<link rel="alternate" type="application/atom+xml" title="$desc (Atom)" href="$atomurl" />};
 			}
 		}
 	}
 	
-	clear_inline_content_cache();
-
 	return $ret if $raw || $nested;
 	push @inline, $ret;
 	return "<div class=\"inline\" id=\"$#inline\"></div>\n\n";
@@ -511,49 +438,25 @@ sub pagetemplate_inline (@) {
 		if exists $feedlinks{$page} && $template->query(name => "feedlinks");
 }
 
-{
-my %inline_content;
-my $cached_destpage="";
-
 sub get_inline_content ($$) {
 	my $page=shift;
 	my $destpage=shift;
 	
-	if (exists $inline_content{$page} && $cached_destpage eq $destpage) {
-		return $inline_content{$page};
-	}
-
 	my $file=$pagesources{$page};
 	my $type=pagetype($file);
-	my $ret="";
 	if (defined $type) {
 		$nested++;
-		$ret=htmlize($page, $destpage, $type,
+		my $ret=htmlize($page, $destpage, $type,
 		       linkify($page, $destpage,
 		       preprocess($page, $destpage,
 		       filter($page, $destpage,
 		       readfile(srcfile($file))))));
 		$nested--;
-		if (isinternal($page)) {
-			# make inlined text of internal pages searchable
-			run_hooks(indexhtml => sub {
-				shift->(page => $page, destpage => $destpage,
-					content => $ret);
-			});
-		}
+		return $ret;
 	}
-	
-	if ($cached_destpage ne $destpage) {
-		clear_inline_content_cache();
-		$cached_destpage=$destpage;
+	else {
+		return "";
 	}
-	return $inline_content{$page}=$ret;
-}
-
-sub clear_inline_content_cache () {
-	%inline_content=();
-}
-
 }
 
 sub date_822 ($) {
@@ -566,88 +469,42 @@ sub date_822 ($) {
 	return $ret;
 }
 
+sub date_3339 ($) {
+	my $time=shift;
+
+	my $lc_time=POSIX::setlocale(&POSIX::LC_TIME);
+	POSIX::setlocale(&POSIX::LC_TIME, "C");
+	my $ret=POSIX::strftime("%Y-%m-%dT%H:%M:%SZ", gmtime($time));
+	POSIX::setlocale(&POSIX::LC_TIME, $lc_time);
+	return $ret;
+}
+
 sub absolute_urls ($$) {
-	# needed because rss sucks
-	my $html=shift;
+	# sucky sub because rss sucks
+	my $content=shift;
 	my $baseurl=shift;
 
 	my $url=$baseurl;
 	$url=~s/[^\/]+$//;
-	my $urltop; # calculated if needed
 
-	my $ret="";
+        # what is the non path part of the url?
+        my $top_uri = URI->new($url);
+        $top_uri->path_query(""); # reset the path
+        my $urltop = $top_uri->as_string;
 
-	eval q{use HTML::Parser; use HTML::Tagset};
-	die $@ if $@;
-	my $p = HTML::Parser->new(api_version => 3);
-	$p->handler(default => sub { $ret.=join("", @_) }, "text");
-	$p->handler(start => sub {
-		my ($tagname, $pos, $text) = @_;
-		if (ref $HTML::Tagset::linkElements{$tagname}) {
-			while (4 <= @$pos) {
-				# use attribute sets from right to left
-				# to avoid invalidating the offsets
-				# when replacing the values
-				my ($k_offset, $k_len, $v_offset, $v_len) =
-					splice(@$pos, -4);
-				my $attrname = lc(substr($text, $k_offset, $k_len));
-				next unless grep { $_ eq $attrname } @{$HTML::Tagset::linkElements{$tagname}};
-				next unless $v_offset; # 0 v_offset means no value
-				my $v = substr($text, $v_offset, $v_len);
-				$v =~ s/^([\'\"])(.*)\1$/$2/;
-				eval q{use HTML::Entities};
-				my $dv = decode_entities($v);
-				if ($dv=~/^#/) {
-					$v=$baseurl.$v; # anchor
-				}
-				elsif ($dv=~/^(?!\w+:)[^\/]/) {
-					$v=$url.$v; # relative url
-				}
-				elsif ($dv=~/^\//) {
-					if (! defined $urltop) {
-						# what is the non path part of the url?
-						my $top_uri = URI->new($url);
-						$top_uri->path_query(""); # reset the path
-						$urltop = $top_uri->as_string;
-					}
-					$v=$urltop.$v; # url relative to top of site
-				}
-				$v =~ s/\"/&quot;/g; # since we quote with ""
-				substr($text, $v_offset, $v_len) = qq("$v");
-			}
-		}
-		$ret.=$text;
-	}, "tagname, tokenpos, text");
-	$p->parse($html);
-	$p->eof;
-
-	return $ret;
-}
-
-sub genenclosure {
-	my $itemtemplate=shift;
-	my $url=shift;
-	my $file=shift;
-
-	return unless $itemtemplate->query(name => "enclosure");
-
-	my $size=(srcfile_stat($file))[8];
-	my $mime="unknown";
-	eval q{use File::MimeInfo};
-	if (! $@) {
-		$mime = mimetype($file);
-	}
-	$itemtemplate->param(
-		enclosure => $url,
-		type => $mime,
-		length => $size,
-	);
+	$content=~s/(<a(?:\s+(?:class|id)\s*="?\w+"?)?)\s+href=\s*"(#[^"]+)"/$1 href="$baseurl$2"/mig;
+        # relative to another wiki page
+	$content=~s/(<a(?:\s+(?:class|id)\s*="?\w+"?)?)\s+href=\s*"(?!\w+:)([^\/][^"]*)"/$1 href="$url$2"/mig;
+	$content=~s/(<img(?:\s+(?:class|id|width|height)\s*="?\w+"?)*)\s+src=\s*"(?!\w+:)([^\/][^"]*)"/$1 src="$url$2"/mig;
+        # relative to the top of the site
+	$content=~s/(<a(?:\s+(?:class|id)\s*="?\w+"?)?)\s+href=\s*"(?!\w+:)(\/[^"]*)"/$1 href="$urltop$2"/mig;
+	$content=~s/(<img(?:\s+(?:class|id|width|height)\s*="?\w+"?)*)\s+src=\s*"(?!\w+:)(\/[^"]*)"/$1 src="$urltop$2"/mig;
+	return $content;
 }
 
 sub genfeed ($$$$$@) {
 	my $feedtype=shift;
 	my $feedurl=shift;
-        my $feedtitle=shift;
 	my $feeddesc=shift;
 	my $guid=shift;
 	my $page=shift;
@@ -655,13 +512,12 @@ sub genfeed ($$$$$@) {
 	
 	my $url=URI->new(encode_utf8(urlto($page,"",1)));
 	
-	my $itemtemplate=template_depends($feedtype."item.tmpl", $page, blind_cache => 1);
+	my $itemtemplate=template($feedtype."item.tmpl", blind_cache => 1);
 	my $content="";
 	my $lasttime = 0;
 	foreach my $p (@pages) {
 		my $u=URI->new(encode_utf8(urlto($p, "", 1)));
 		my $pcontent = absolute_urls(get_inline_content($p, $page), $url);
-		my $fancy_enclosure_seen = 0;
 
 		$itemtemplate->param(
 			title => pagetitle(basename($p)),
@@ -675,31 +531,39 @@ sub genfeed ($$$$$@) {
 
 		if (exists $pagestate{$p}) {
 			if (exists $pagestate{$p}{meta}{guid}) {
-				eval q{use HTML::Entities};
-				$itemtemplate->param(guid => HTML::Entities::encode_numeric($pagestate{$p}{meta}{guid}));
+				$itemtemplate->param(guid => $pagestate{$p}{meta}{guid});
 			}
 
 			if (exists $pagestate{$p}{meta}{updated}) {
 				$itemtemplate->param(mdate_822 => date_822($pagestate{$p}{meta}{updated}));
 				$itemtemplate->param(mdate_3339 => date_3339($pagestate{$p}{meta}{updated}));
 			}
+		}
 
-			if (exists $pagestate{$p}{meta}{enclosure}) {
-				my $absurl = $pagestate{$p}{meta}{enclosure};
-				my $file = $pagestate{$p}{meta}{enclosurefile};
-				genenclosure($itemtemplate, $absurl, $file);
-				$fancy_enclosure_seen = 1;
+		if ($itemtemplate->query(name => "enclosure")) {
+			my $file=$pagesources{$p};
+			my $type=pagetype($file);
+			if (defined $type) {
+				$itemtemplate->param(content => $pcontent);
+			}
+			else {
+				my $size=(srcfile_stat($file))[8];
+				my $mime="unknown";
+				eval q{use File::MimeInfo};
+				if (! $@) {
+					$mime = mimetype($file);
+				}
+				$itemtemplate->param(
+					enclosure => $u,
+					type => $mime,
+					length => $size,
+				);
 			}
 		}
-
-		my $file=$pagesources{$p};
-		unless ($fancy_enclosure_seen || defined(pagetype($file))) {
-			genenclosure($itemtemplate, $u, $file);
-			$itemtemplate->param(simplepodcast => 1);
+		else {
+			$itemtemplate->param(content => $pcontent);
 		}
 
-		$itemtemplate->param(content => $pcontent);
-
 		run_hooks(pagetemplate => sub {
 			shift->(page => $p, destpage => $page,
 				template => $itemtemplate);
@@ -711,17 +575,17 @@ sub genfeed ($$$$$@) {
 		$lasttime = $pagemtime{$p} if $pagemtime{$p} > $lasttime;
 	}
 
-	my $template=template_depends($feedtype."page.tmpl", $page, blind_cache => 1);
+	my $template=template($feedtype."page.tmpl", blind_cache => 1);
 	$template->param(
-		title => $feedtitle,
+		title => $page ne "index" ? pagetitle($page) : $config{wikiname},
 		wikiname => $config{wikiname},
 		pageurl => $url,
 		content => $content,
 		feeddesc => $feeddesc,
 		guid => $guid,
 		feeddate => date_3339($lasttime),
-		feeddate_822 => date_822($lasttime),
 		feedurl => $feedurl,
+		version => $IkiWiki::version,
 	);
 	run_hooks(pagetemplate => sub {
 		shift->(page => $page, destpage => $page,
