@@ -20,7 +20,6 @@ sub getsetup () {
 		plugin => {
 			safe => 1,
 			rebuild => undef,
-			section => "core",
 		},
 }
 
@@ -39,10 +38,10 @@ sub needsbuild (@) {
 	}
 }
 
-sub scrub ($$$) {
+sub scrub ($$) {
 	if (IkiWiki::Plugin::htmlscrubber->can("sanitize")) {
 		return IkiWiki::Plugin::htmlscrubber::sanitize(
-			content => shift, page => shift, destpage => shift);
+			content => shift, destpage => shift);
 	}
 	else {
 		return shift;
@@ -88,21 +87,15 @@ sub preprocess (@) {
 
 	# Metadata collection that needs to happen during the scan pass.
 	if ($key eq 'title') {
-		$pagestate{$page}{meta}{title}=$value;
-		if (exists $params{sortas}) {
-			$pagestate{$page}{meta}{titlesort}=$params{sortas};
-		}
-		else {
-			delete $pagestate{$page}{meta}{titlesort};
-		}
-		return "";
+		$pagestate{$page}{meta}{title}=HTML::Entities::encode_numeric($value);
+		# fallthrough
 	}
 	elsif ($key eq 'description') {
-		$pagestate{$page}{meta}{description}=$value;
+		$pagestate{$page}{meta}{description}=HTML::Entities::encode_numeric($value);
 		# fallthrough
 	}
 	elsif ($key eq 'guid') {
-		$pagestate{$page}{meta}{guid}=$value;
+		$pagestate{$page}{meta}{guid}=HTML::Entities::encode_numeric($value);
 		# fallthrough
 	}
 	elsif ($key eq 'license') {
@@ -122,22 +115,12 @@ sub preprocess (@) {
 	}
 	elsif ($key eq 'author') {
 		$pagestate{$page}{meta}{author}=$value;
-		if (exists $params{sortas}) {
-			$pagestate{$page}{meta}{authorsort}=$params{sortas};
-		}
-		else {
-			delete $pagestate{$page}{meta}{authorsort};
-		}
 		# fallthorough
 	}
 	elsif ($key eq 'authorurl') {
 		$pagestate{$page}{meta}{authorurl}=$value if safeurl($value);
 		# fallthrough
 	}
-	elsif ($key eq 'permalink') {
-		$pagestate{$page}{meta}{permalink}=$value if safeurl($value);
-		# fallthrough
-	}
 	elsif ($key eq 'date') {
 		eval q{use Date::Parse};
 		if (! $@) {
@@ -158,10 +141,11 @@ sub preprocess (@) {
 		return;
 	}
 
-	# Metadata handling that happens only during preprocessing pass.
+	# Metadata collection that happens only during preprocessing pass.
 	if ($key eq 'permalink') {
 		if (safeurl($value)) {
-			push @{$metaheaders{$page}}, scrub('<link rel="bookmark" href="'.encode_entities($value).'" />', $page, $destpage);
+			$pagestate{$page}{meta}{permalink}=$value;
+			push @{$metaheaders{$page}}, scrub('<link rel="bookmark" href="'.encode_entities($value).'" />', $destpage);
 		}
 	}
 	elsif ($key eq 'stylesheet') {
@@ -173,10 +157,10 @@ sub preprocess (@) {
 		if (! length $stylesheet) {
 			error gettext("stylesheet not found")
 		}
-		push @{$metaheaders{$page}}, scrub('<link href="'.urlto($stylesheet, $page).
+		push @{$metaheaders{$page}}, '<link href="'.urlto($stylesheet, $page).
 			'" rel="'.encode_entities($rel).
 			'" title="'.encode_entities($title).
-			"\" type=\"text/css\" />", $page, $destpage);
+			"\" type=\"text/css\" />";
 	}
 	elsif ($key eq 'openid') {
 		my $delegate=0; # both by default
@@ -211,7 +195,7 @@ sub preprocess (@) {
 			if (! length $link) {
 				error gettext("redir page not found")
 			}
-			add_depends($page, $link, deptype("presence"));
+			add_depends($page, $link);
 
 			$value=urlto($link, $page);
 			$value.='#'.$redir_anchor if defined $redir_anchor;
@@ -235,7 +219,7 @@ sub preprocess (@) {
 		my $delay=int(exists $params{delay} ? $params{delay} : 0);
 		my $redir="<meta http-equiv=\"refresh\" content=\"$delay; URL=$value\" />";
 		if (! $safe) {
-			$redir=scrub($redir, $page, $destpage);
+			$redir=scrub($redir, $destpage);
 		}
 		push @{$metaheaders{$page}}, $redir;
 	}
@@ -245,28 +229,16 @@ sub preprocess (@) {
 				join(" ", map {
 					encode_entities($_)."=\"".encode_entities(decode_entities($params{$_}))."\""
 				} keys %params).
-				" />\n", $page, $destpage);
+				" />\n", $destpage);
 		}
 	}
 	elsif ($key eq 'robots') {
 		push @{$metaheaders{$page}}, '<meta name="robots"'.
 			' content="'.encode_entities($value).'" />';
 	}
-	elsif ($key eq 'description') {
-		push @{$metaheaders{$page}}, '<meta name="'.
-			encode_entities($key).
-			'" content="'.encode_entities($value).'" />';
-	}
-	elsif ($key eq 'name') {
-		push @{$metaheaders{$page}}, scrub('<meta '.$key.'="'.
-			encode_entities($value).
-			join(' ', map { "$_=\"$params{$_}\"" } keys %params).
-			' />', $page, $destpage);
-	}
 	else {
-		push @{$metaheaders{$page}}, scrub('<meta name="'.
-			encode_entities($key).'" content="'.
-			encode_entities($value).'" />', $page, $destpage);
+		push @{$metaheaders{$page}}, scrub('<meta name="'.encode_entities($key).
+			'" content="'.encode_entities($value).'" />', $destpage);
 	}
 
 	return "";
@@ -284,20 +256,12 @@ sub pagetemplate (@) {
 		$template->param(meta => join("\n", grep { (! $seen{$_}) && ($seen{$_}=1) } @{$metaheaders{$page}}));
 	}
 	if (exists $pagestate{$page}{meta}{title} && $template->query(name => "title")) {
-		eval q{use HTML::Entities};
-		$template->param(title => HTML::Entities::encode_numeric($pagestate{$page}{meta}{title}));
+		$template->param(title => $pagestate{$page}{meta}{title});
 		$template->param(title_overridden => 1);
 	}
 
-	foreach my $field (qw{authorurl permalink}) {
-		eval q{use HTML::Entities};
-		$template->param($field => HTML::Entities::encode_entities($pagestate{$page}{meta}{$field}))
-			if exists $pagestate{$page}{meta}{$field} && $template->query(name => $field);
-	}
-
-	foreach my $field (qw{description author}) {
-		eval q{use HTML::Entities};
-		$template->param($field => HTML::Entities::encode_numeric($pagestate{$page}{meta}{$field}))
+	foreach my $field (qw{author authorurl permalink}) {
+		$template->param($field => $pagestate{$page}{meta}{$field})
 			if exists $pagestate{$page}{meta}{$field} && $template->query(name => $field);
 	}
 
@@ -310,33 +274,6 @@ sub pagetemplate (@) {
 	}
 }
 
-sub get_sort_key {
-	my $page = shift;
-	my $meta = shift;
-
-	# e.g. titlesort (also makes sense for author)
-	my $key = $pagestate{$page}{meta}{$meta . "sort"};
-	return $key if defined $key;
-
-	# e.g. title
-	$key = $pagestate{$page}{meta}{$meta};
-	return $key if defined $key;
-
-	# fall back to closer-to-core things
-	if ($meta eq 'title') {
-		return pagetitle(IkiWiki::basename($page));
-	}
-	elsif ($meta eq 'date') {
-		return $IkiWiki::pagectime{$page};
-	}
-	elsif ($meta eq 'updated') {
-		return $IkiWiki::pagemtime{$page};
-	}
-	else {
-		return '';
-	}
-}
-
 sub match {
 	my $field=shift;
 	my $page=shift;
@@ -354,21 +291,21 @@ sub match {
 
 	if (defined $val) {
 		if ($val=~/^$re$/i) {
-			return IkiWiki::SuccessReason->new("$re matches $field of $page", $page => $IkiWiki::DEPEND_CONTENT, "" => 1);
+			return IkiWiki::SuccessReason->new("$re matches $field of $page");
 		}
 		else {
-			return IkiWiki::FailReason->new("$re does not match $field of $page", $page => $IkiWiki::DEPEND_CONTENT, "" => 1);
+			return IkiWiki::FailReason->new("$re does not match $field of $page");
 		}
 	}
 	else {
-		return IkiWiki::FailReason->new("$page does not have a $field", $page => $IkiWiki::DEPEND_CONTENT);
+		return IkiWiki::FailReason->new("$page does not have a $field");
 	}
 }
 
 package IkiWiki::PageSpec;
 
 sub match_title ($$;@) {
-	IkiWiki::Plugin::meta::match("title", @_);
+	IkiWiki::Plugin::meta::match("title", @_);	
 }
 
 sub match_author ($$;@) {
@@ -387,31 +324,4 @@ sub match_copyright ($$;@) {
 	IkiWiki::Plugin::meta::match("copyright", @_);
 }
 
-sub match_guid ($$;@) {
-	IkiWiki::Plugin::meta::match("guid", @_);
-}
-
-package IkiWiki::SortSpec;
-
-sub cmp_meta {
-	my $meta = shift;
-	error(gettext("sort=meta requires a parameter")) unless defined $meta;
-
-	if ($meta eq 'updated' || $meta eq 'date') {
-		return IkiWiki::Plugin::meta::get_sort_key($a, $meta)
-			<=>
-			IkiWiki::Plugin::meta::get_sort_key($b, $meta);
-	}
-
-	return IkiWiki::Plugin::meta::get_sort_key($a, $meta)
-		cmp
-		IkiWiki::Plugin::meta::get_sort_key($b, $meta);
-}
-
-# A prototype of how sort=title could behave in 4.0 or something
-sub cmp_meta_title {
-	$_[0] = 'title';
-	return cmp_meta(@_);
-}
-
 1
