@@ -15,7 +15,6 @@ sub getsetup () {
 		plugin => {
 			safe => 1,
 			rebuild => 1,
-			section => "web",
 		},
 		mirrorlist => {
 			type => "string",
@@ -30,8 +29,7 @@ sub pagetemplate (@) {
 	my %params=@_;
         my $template=$params{template};
 	
-	if ($template->query(name => "extrafooter") &&
-	    keys %{$config{mirrorlist}} > 0) {
+	if ($template->query(name => "extrafooter")) {
 		my $value=$template->param("extrafooter");
 		$value.=mirrorlist($params{page});
 		$template->param(extrafooter => $value);
@@ -40,7 +38,7 @@ sub pagetemplate (@) {
 
 sub mirrorlist ($) {
 	my $page=shift;
-	return ($config{html5} ? '<nav id="mirrorlist">' : '<div>').
+	return "<p>".
 		(keys %{$config{mirrorlist}} > 1 ? gettext("Mirrors") : gettext("Mirror")).
 		": ".
 		join(", ",
@@ -50,7 +48,7 @@ sub mirrorlist ($) {
 				qq{">$_</a>}
 			} keys %{$config{mirrorlist}}
 		).
-		($config{html5} ? '</nav>' : '</div>');
+		"</p>";
 }
 
 1
