@@ -23,7 +23,6 @@ sub import {
 	hook(type => "rcs", id => "rcs_recentchanges", call => \&rcs_recentchanges);
 	hook(type => "rcs", id => "rcs_diff", call => \&rcs_diff);
 	hook(type => "rcs", id => "rcs_getctime", call => \&rcs_getctime);
-	hook(type => "rcs", id => "rcs_getmtime", call => \&rcs_getmtime);
 }
 
 sub checkconfig () {
@@ -69,7 +68,6 @@ sub getsetup () {
 		plugin => {
 			safe => 0, # rcs plugin
 			rebuild => undef,
-			section => "rcs",
 		},
 		mtn_wrapper => {
 			type => "string",
@@ -230,7 +228,7 @@ sub read_certs ($$) {
 	my @ret;
 
 	my $line = $results[0];
-	while ($line =~ m/\s+key\s["\[](.*?)[\]"]\nsignature\s"(ok|bad|unknown)"\n\s+name\s"(.*?)"\n\s+value\s"(.*?)"\n\s+trust\s"(trusted|untrusted)"\n/sg) {
+	while ($line =~ m/\s+key\s"(.*?)"\nsignature\s"(ok|bad|unknown)"\n\s+name\s"(.*?)"\n\s+value\s"(.*?)"\n\s+trust\s"(trusted|untrusted)"\n/sg) {
 		push @ret, {
 			key => $1,
 			signature => $2,
@@ -293,33 +291,31 @@ sub rcs_prepedit ($) {
 	return get_rev();
 }
 
-sub commitauthor (@) {
-	my %params=@_;
-
-	if (defined $params{session}) {
-		if (defined $params{session}->param("name")) {
-			return "Web user: " . $params{session}->param("name");
-		}
-		elsif (defined $params{session}->remote_addr()) {
-			return "Web IP: " . $params{session}->remote_addr();
-		}
-	}
-	return "Web: Anonymous";
-}
-
-
-sub rcs_commit (@) {
+sub rcs_commit ($$$;$$) {
 	# Tries to commit the page; returns undef on _success_ and
 	# a version of the page with the rcs's conflict markers on failure.
 	# The file is relative to the srcdir.
-	my %params=@_;
+	my $file=shift;
+	my $message=shift;
+	my $rcstoken=shift;
+	my $user=shift;
+	my $ipaddr=shift;
+	my $author;
 
-	my $author=IkiWiki::possibly_foolish_untaint(commitauthor(%params)),
+	if (defined $user) {
+		$author="Web user: " . $user;
+	}
+	elsif (defined $ipaddr) {
+		$author="Web IP: " . $ipaddr;
+	}
+	else {
+		$author="Web: Anonymous";
+	}
 
 	chdir $config{srcdir}
 	    or error("Cannot chdir to $config{srcdir}: $!");
 
-	my ($oldrev) = $params{token} =~ m/^($sha1_pattern)$/; # untaint
+	my ($oldrev)= $rcstoken=~ m/^($sha1_pattern)$/; # untaint
 	my $rev = get_rev();
 	if (defined $rev && defined $oldrev && $rev ne $oldrev) {
 		my $automator = Monotone->new();
@@ -328,8 +324,8 @@ sub rcs_commit (@) {
 		# Something has been committed, has this file changed?
 		my ($out, $err);
 		$automator->setOpts("r", $oldrev, "r", $rev);
-		($out, $err) = $automator->call("content_diff", $params{file});
-		debug("Problem committing $params{file}") if ($err ne "");
+		($out, $err) = $automator->call("content_diff", $file);
+		debug("Problem committing $file") if ($err ne "");
 		my $diff = $out;
 		
 		if ($diff) {
@@ -338,11 +334,11 @@ sub rcs_commit (@) {
 			#
 			# first get the contents
 			debug("File changed: forming branch");
-			my $newfile=readfile("$config{srcdir}/$params{file}");
+			my $newfile=readfile("$config{srcdir}/$file");
 			
 			# then get the old content ID from the diff
-			if ($diff !~ m/^---\s$params{file}\s+($sha1_pattern)$/m) {
-				error("Unable to find previous file ID for $params{file}");
+			if ($diff !~ m/^---\s$file\s+($sha1_pattern)$/m) {
+				error("Unable to find previous file ID for $file");
 			}
 			my $oldFileID = $1;
 
@@ -353,13 +349,13 @@ sub rcs_commit (@) {
 			my $branch = $1;
 
 			# then put the new content into the DB (and record the new content ID)
-			my $newRevID = commit_file_to_new_rev($automator, $params{file}, $oldFileID, $newfile, $oldrev, $branch, $author, $params{message});
+			my $newRevID = commit_file_to_new_rev($automator, $file, $oldFileID, $newfile, $oldrev, $branch, $author, $message);
 
 			$automator->close();
 
 			# if we made it to here then the file has been committed... revert the local copy
-			if (system("mtn", "--root=$config{mtnrootdir}", "revert", $params{file}) != 0) {
-				debug("Unable to revert $params{file} after merge on conflicted commit!");
+			if (system("mtn", "--root=$config{mtnrootdir}", "revert", $file) != 0) {
+				debug("Unable to revert $file after merge on conflicted commit!");
 			}
 			debug("Divergence created! Attempting auto-merge.");
 
@@ -408,7 +404,7 @@ sub rcs_commit (@) {
 				# for cleanup note, this relies on the fact
 				# that ikiwiki seems to call rcs_prepedit()
 				# again after we return
-				return readfile("$config{srcdir}/$params{file}");
+				return readfile("$config{srcdir}/$file");
 			}
 			return undef;
 		}
@@ -420,12 +416,11 @@ sub rcs_commit (@) {
 
 	if (system("mtn", "--root=$config{mtnrootdir}", "commit", "--quiet",
 	           "--author", $author, "--key", $config{mtnkey}, "-m",
-	           IkiWiki::possibly_foolish_untaint($params{message}),
-	           $params{file}) != 0) {
+		   IkiWiki::possibly_foolish_untaint($message), $file) != 0) {
 		debug("Traditional commit failed! Returning data as conflict.");
-		my $conflict=readfile("$config{srcdir}/$params{file}");
+		my $conflict=readfile("$config{srcdir}/$file");
 		if (system("mtn", "--root=$config{mtnrootdir}", "revert",
-		           "--quiet", $params{file}) != 0) {
+		           "--quiet", $file) != 0) {
 			debug("monotone revert failed");
 		}
 		return $conflict;
@@ -441,21 +436,32 @@ sub rcs_commit (@) {
 	return undef # success
 }
 
-sub rcs_commit_staged (@) {
+sub rcs_commit_staged ($$$) {
 	# Commits all staged changes. Changes can be staged using rcs_add,
 	# rcs_remove, and rcs_rename.
-	my %params=@_;
-
+	my ($message, $user, $ipaddr)=@_;
+	
 	# Note - this will also commit any spurious changes that happen to be
 	# lying around in the working copy.  There shouldn't be any, but...
 	
 	chdir $config{srcdir}
 	    or error("Cannot chdir to $config{srcdir}: $!");
 
+	my $author;
+
+	if (defined $user) {
+		$author="Web user: " . $user;
+	}
+	elsif (defined $ipaddr) {
+		$author="Web IP: " . $ipaddr;
+	}
+	else {
+		$author="Web: Anonymous";
+	}
+
 	if (system("mtn", "--root=$config{mtnrootdir}", "commit", "--quiet",
-	           "--author", IkiWiki::possibly_foolish_untaint(commitauthor(%params)),
-	           "--key", $config{mtnkey}, "-m",
-		   IkiWiki::possibly_foolish_untaint($params{message})) != 0) {
+	           "--author", $author, "--key", $config{mtnkey}, "-m",
+		   IkiWiki::possibly_foolish_untaint($message)) != 0) {
 		error("Monotone commit failed");
 	}
 }
@@ -552,8 +558,7 @@ sub rcs_recentchanges ($) {
 					# from the changelog
 					if ($cert->{key} eq $config{mtnkey}) {
 						$committype = "web";
-					}
-					else {
+					} else {
 						$committype = "mtn";
 					}
 				} elsif ($cert->{name} eq "date") {
@@ -686,8 +691,4 @@ sub rcs_getctime ($) {
 	return $date;
 }
 
-sub rcs_getmtime ($) {
-	error "rcs_getmtime is not implemented for monotone\n"; # TODO
-}
-
 1
