@@ -4,45 +4,61 @@ package IkiWiki::Plugin::openid;
 
 use warnings;
 use strict;
-use IkiWiki 2.00;
+use IkiWiki 3.00;
 
-sub import { #{{{
+sub import {
 	hook(type => "getopt", id => "openid", call => \&getopt);
+	hook(type => "getsetup", id => "openid", call => \&getsetup);
 	hook(type => "auth", id => "openid", call => \&auth);
 	hook(type => "formbuilder_setup", id => "openid",
 		call => \&formbuilder_setup, last => 1);
-} # }}}
+}
 
-sub getopt () { #{{{
+sub getopt () {
 	eval q{use Getopt::Long};
 	error($@) if $@;
 	Getopt::Long::Configure('pass_through');
 	GetOptions("openidsignup=s" => \$config{openidsignup});
-} #}}}
+}
 
-sub formbuilder_setup (@) { #{{{
+sub getsetup () {
+	return
+		plugin => {
+			safe => 1,
+			rebuild => 0,
+		},
+		openidsignup => {
+			type => "string",
+			example => "http://myopenid.com/",
+			description => "an url where users can signup for an OpenID",
+			safe => 1,
+			rebuild => 0,
+		},
+}
+
+sub formbuilder_setup (@) {
 	my %params=@_;
 
 	my $form=$params{form};
 	my $session=$params{session};
 	my $cgi=$params{cgi};
 	
-	# Give up if module is unavailable to avoid needing to depend on
-	# it.
-	eval q{use Net::OpenID::Consumer};
-	if ($@) {
-		debug("unable to load Net::OpenID::Consumer, not enabling OpenID login");
-		return;
-	}
-
 	if ($form->title eq "signin") {
+		# Give up if module is unavailable to avoid
+		# needing to depend on it.
+		eval q{use Net::OpenID::Consumer};
+		if ($@) {
+			debug("unable to load Net::OpenID::Consumer, not enabling OpenID login ($@)");
+			return;
+		}
+
 		# This avoids it displaying a redundant label for the
 		# OpenID fieldset.
 		$form->fieldsets("OpenID");
 
  		$form->field(
 			name => "openid_url",
-			label => gettext("Log in with")." ".htmllink("", "", "OpenID", noimageinline => 1),
+			label => gettext("Log in with")." ".htmllink("", "", "ikiwiki/OpenID", noimageinline => 1),
 			fieldset => "OpenID",
 			size => 30,
 			comment => ($config{openidsignup} ? " | <a href=\"$config{openidsignup}\">".gettext("Get an OpenID")."</a>" : "")
@@ -76,7 +92,7 @@ sub formbuilder_setup (@) { #{{{
 	}
 }
 
-sub validate ($$$;$) { #{{{
+sub validate ($$$;$) {
 	my $q=shift;
 	my $session=shift;
 	my $openid_url=shift;
@@ -105,9 +121,9 @@ sub validate ($$$;$) { #{{{
 	# eventually bounce them back to auth()
 	IkiWiki::redirect($q, $check_url);
 	exit 0;
-} #}}}
+}
 
-sub auth ($$) { #{{{
+sub auth ($$) {
 	my $q=shift;
 	my $session=shift;
 
@@ -131,9 +147,9 @@ sub auth ($$) { #{{{
 		# myopenid.com affiliate support
 		validate($q, $session, $q->param('openid_identifier'));
 	}
-} #}}}
+}
 
-sub getobj ($$) { #{{{
+sub getobj ($$) {
 	my $q=shift;
 	my $session=shift;
 
@@ -162,6 +178,6 @@ sub getobj ($$) { #{{{
 		consumer_secret => sub { return shift()+$secret },
 		required_root => $config{cgiurl},
 	);
-} #}}}
+}
 
 1
