@@ -3,14 +3,23 @@ package IkiWiki::Plugin::postsparkline;
 
 use warnings;
 use strict;
-use IkiWiki 2.00;
+use IkiWiki 3.00;
 
-sub import { #{{{
+sub import {
 	IkiWiki::loadplugin('sparkline');
+	hook(type => "getsetup", id => "postsparkline", call => \&getsetup);
 	hook(type => "preprocess", id => "postsparkline", call => \&preprocess);
-} # }}}
+}
 
-sub preprocess (@) { #{{{
+sub getsetup () {
+	return 
+		plugin => {
+			safe => 1,
+			rebuild => undef,
+		},
+}
+
+sub preprocess (@) {
 	my %params=@_;
 
 	if (! exists $params{max}) {
@@ -21,38 +30,50 @@ sub preprocess (@) { #{{{
 		return "";
 	}
 
+	if (! exists $params{time} || $params{time} ne 'mtime') {
+		$params{timehash} = \%IkiWiki::pagectime;
+	}
+	else {
+		$params{timehash} = \%IkiWiki::pagemtime;
+	}
+
 	if (! exists $params{formula}) {
-		return "[[postsparkline ".gettext("missing formula")."]]";
+		error gettext("missing formula")
 	}
 	my $formula=$params{formula};
 	$formula=~s/[^a-zA-Z0-9]*//g;
 	$formula=IkiWiki::possibly_foolish_untaint($formula);
 	if (! length $formula ||
 	    ! IkiWiki::Plugin::postsparkline::formula->can($formula)) {
-		return "[[postsparkline ".gettext("unknown formula")."]]";
+		error gettext("unknown formula");
 	}
 
 	add_depends($params{page}, $params{pages});
 
-	my @list;
-	foreach my $page (keys %pagesources) {
-		next if $page eq $params{page};
-		if (pagespec_match($page, $params{pages}, location => $params{page})) {
-			push @list, $page;
-		}
+	my @list=sort { $params{timehash}->{$b} <=> $params{timehash}->{$a} } 
+		pagespec_match_list(
+			[ grep { $_ ne $params{page} } keys %pagesources],
+			$params{pages}, location => $params{page});
+
+	my @data=eval qq{IkiWiki::Plugin::postsparkline::formula::$formula(\\\%params, \@list)};
+	if ($@) {
+		error $@;
 	}
-	
-	@list = sort { $IkiWiki::pagectime{$b} <=> $IkiWiki::pagectime{$a} } @list;
+
+	if (! @data) {
+		# generate an empty graph
+		push @data, 0 foreach 1..($params{max} / 2);
+	}
+
+	my $color=exists $params{color} ? "($params{color})" : "";
 
 	delete $params{pages};
 	delete $params{formula};
-	my @data=eval qq{IkiWiki::Plugin::postsparkline::formula::$formula(\\\%params, \@list)};
-	if ($@) {
-		return "[[postsparkline error $@]]";
-	}
+	delete $params{ftime};
+	delete $params{color};
 	return IkiWiki::Plugin::sparkline::preprocess(%params, 
-		map { $_ => "" } reverse @data);
-} # }}}
+		map { $_.$color => "" } reverse @data);
+}
 
 sub perfoo ($@) {
 	my $sub=shift;
@@ -63,7 +84,7 @@ sub perfoo ($@) {
 	my $count=0;
 	my @data;
 	foreach (@_) {
-		$cur=$sub->($IkiWiki::pagectime{$_});
+		$cur=$sub->($params->{timehash}->{$_});
 		if (defined $prev) {
 			if ($prev != $cur) {
 				push @data, "$prev,$count";
@@ -114,7 +135,7 @@ sub interval ($@) {
 	my $max=$params->{max};
 	my @data;
 	for (my $i=1; $i < @_; $i++) {
-		push @data, $IkiWiki::pagectime{$_[$i-1]} - $IkiWiki::pagectime{$_[$i]};
+		push @data, $params->{timehash}->{$_[$i-1]} - $params->{timehash}->{$_[$i]};
 		last if --$max <= 0;
 	}
 	return @data;
