@@ -4,30 +4,46 @@ package IkiWiki::Plugin::search;
 
 use warnings;
 use strict;
-use IkiWiki 2.00;
+use IkiWiki 3.00;
 
-sub import { #{{{
+sub import {
+	hook(type => "getsetup", id => "search", call => \&getsetup);
 	hook(type => "checkconfig", id => "search", call => \&checkconfig);
 	hook(type => "pagetemplate", id => "search", call => \&pagetemplate);
-	hook(type => "sanitize", id => "search", call => \&index);
+	hook(type => "postscan", id => "search", call => \&index);
 	hook(type => "delete", id => "search", call => \&delete);
 	hook(type => "cgi", id => "search", call => \&cgi);
-} # }}}
+}
 
-sub checkconfig () { #{{{
+sub getsetup () {
+	return
+		plugin => {
+			safe => 1,
+			rebuild => 1,
+		},
+		omega_cgi => {
+			type => "string",
+			example => "/usr/lib/cgi-bin/omega/omega",
+			description => "path to the omega cgi program",
+			safe => 0, # external program
+			rebuild => 0,
+		},
+}
+
+sub checkconfig () {
 	foreach my $required (qw(url cgiurl)) {
 		if (! length $config{$required}) {
-			error(sprintf(gettext("Must specify %s when using the search plugin"), $required));
+			error(sprintf(gettext("Must specify %s when using the %s plugin"), $required, 'search'));
 		}
 	}
 	
-	if (! exists $config{omega_cgi}) {
+	if (! defined $config{omega_cgi}) {
 		$config{omega_cgi}="/usr/lib/cgi-bin/omega/omega";
 	}
-} #}}}
+}
 
 my $form;
-sub pagetemplate (@) { #{{{
+sub pagetemplate (@) {
 	my %params=@_;
 	my $page=$params{page};
 	my $template=$params{template};
@@ -42,14 +58,12 @@ sub pagetemplate (@) { #{{{
 
 		$template->param(searchform => $form);
 	}
-} #}}}
+}
 
 my $scrubber;
 my $stemmer;
-sub index (@) { #{{{
+sub index (@) {
 	my %params=@_;
-	
-	return $params{content} if $IkiWiki::preprocessing{$params{destpage}};
 
 	setupfiles();
 
@@ -59,7 +73,7 @@ sub index (@) { #{{{
 	
 	my $db=xapiandb();
 	my $doc=Search::Xapian::Document->new();
-	my $caption=IkiWiki::pagetitle($params{page});
+	my $caption=pagetitle($params{page});
 	my $title;
 	if (exists $pagestate{$params{page}}{meta} &&
 		exists $pagestate{$params{page}}{meta}{title}) {
@@ -125,26 +139,24 @@ sub index (@) { #{{{
 	$tg->index_text($caption, 2);
 	$tg->index_text($title, 2) if $title ne $caption;
 	$tg->index_text($toindex);
-	$tg->index_text(lc($title), 1, "ZS"); # for title:foo
+	$tg->index_text(lc($title), 1, "S"); # for title:foo
 	foreach my $link (@{$links{$params{page}}}) {
-		$tg->index_text(lc($link), 1, "ZLINK"); # for link:bar
+		$tg->index_text(lc($link), 1, "XLINK"); # for link:bar
 	}
 
 	$doc->add_term($pageterm);
 	$db->replace_document_by_term($pageterm, $doc);
+}
 
-	return $params{content};
-} #}}}
-
-sub delete (@) { #{{{
+sub delete (@) {
 	my $db=xapiandb();
 	foreach my $page (@_) {
 		my $pageterm=pageterm(pagename($page));
 		$db->delete_document_by_term($pageterm) if defined $pageterm;
 	}
-} #}}}
+}
 
-sub cgi ($) { #{{{
+sub cgi ($) {
 	my $cgi=shift;
 
 	if (defined $cgi->param('P')) {
@@ -157,9 +169,9 @@ sub cgi ($) { #{{{
 			noimageinline => 1, linktext => "Help");
 		exec($config{omega_cgi}) || error("$config{omega_cgi} failed: $!");
 	}
-} #}}}
+}
 
-sub pageterm ($) { #{{{
+sub pageterm ($) {
 	my $page=shift;
 
 	# 240 is the number used by omindex to decide when to hash an
@@ -178,10 +190,10 @@ sub pageterm ($) { #{{{
 	else {
 		return "U:".$page;
 	}
-} #}}}
+}
 
 my $db;
-sub xapiandb () { #{{{
+sub xapiandb () {
 	if (! defined $db) {
 		eval q{
 			use Search::Xapian;
@@ -192,11 +204,11 @@ sub xapiandb () { #{{{
 			Search::Xapian::DB_CREATE_OR_OPEN());
 	}
 	return $db;
-} #}}}
+}
 
 {
 my $setup=0;
-sub setupfiles () { #{{{
+sub setupfiles () {
 	if (! $setup and (! -e $config{wikistatedir}."/xapian" || $config{rebuild})) {
 		writefile("omega.conf", $config{wikistatedir}."/xapian",
 			"database_dir .\n".
@@ -206,7 +218,7 @@ sub setupfiles () { #{{{
 				readfile(IkiWiki::template_file("searchquery.tmpl"))));
 		$setup=1;
 	}
-} #}}}
+}
 }
 
 1
