@@ -15,7 +15,6 @@ sub ask ($$) {
 	my ($question, $default)=@_;
 
 	my $r=Term::ReadLine->new("ikiwiki");
-	$r->ornaments("md,me");
 	$r->readline(encode_utf8($question)." ", $default);
 }
 
@@ -38,22 +37,19 @@ sub sanitize_wikiname ($) {
 
 sub import (@) {
 	my $this=shift;
-	$config{setuptype}='Standard';
 	IkiWiki::Setup::merge({@_});
 
-	if (! $config{force_overwrite}) {
-		# Avoid overwriting any existing files.
-		foreach my $key (qw{srcdir destdir repository dumpsetup}) {
-			next unless exists $config{$key};
-			my $add="";
-			my $dir=IkiWiki::dirname($config{$key})."/";
-			my $base=IkiWiki::basename($config{$key});
-			while (-e $dir.$add.$base) {
-				$add=1 if ! $add;
-				$add++;
-			}
-			$config{$key}=$dir.$add.$base;
+	# Avoid overwriting any existing files.
+	foreach my $key (qw{srcdir destdir repository dumpsetup}) {
+		next unless exists $config{$key};
+		my $add="";
+		my $dir=IkiWiki::dirname($config{$key})."/";
+		my $base=IkiWiki::basename($config{$key});
+		while (-e $dir.$add.$base) {
+			$add=1 if ! $add;
+			$add++;
 		}
+		$config{$key}=$dir.$add.$base;
 	}
 	
 	# Set up wrapper
@@ -72,18 +68,9 @@ sub import (@) {
 		}
 		elsif ($config{rcs} eq 'bzr') {
 			# TODO
-			print STDERR "warning: do not know how to set up the bzr_wrapper hook!\n";
 		}
 		elsif ($config{rcs} eq 'mercurial') {
 			# TODO
-			print STDERR "warning: do not know how to set up the mercurial_wrapper hook!\n";
-		}
-		elsif ($config{rcs} eq 'tla') {
-			# TODO
-			print STDERR "warning: do not know how to set up the tla_wrapper hook!\n";
-		}
-		elsif ($config{rcs} eq 'cvs') {
-			$config{cvs_wrapper}=$config{repository}."/CVSROOT/post-commit";
 		}
 		else {
 			error sprintf(gettext("unsupported revision control system %s"),
@@ -130,10 +117,9 @@ sub import (@) {
 				IkiWiki::run_hooks(checkconfig => sub { shift->() });
 			};
 			if ($@) {
-				my $err=$@;
 				print STDERR sprintf(gettext("** Disabling plugin %s, since it is failing with this message:"),
 					$plugin)."\n";
-				print STDERR "$err\n";
+				print STDERR "$@\n";
 				push @{$bakconfig{disable_plugins}}, $plugin;
 			}
 		}
@@ -152,7 +138,7 @@ sub import (@) {
 
 	# Create admin user(s).
 	foreach my $admin (@{$config{adminuser}}) {
-		next if defined IkiWiki::openiduser($admin);
+		next if $admin=~/^http\?:\/\//; # openid
 		
 		# Prompt for password w/o echo.
 		my ($password, $password2);
